@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,25 +13,44 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
-/* Return error-text for system error messages and nisam messages */
+/* Return error-text for system error messages and handler messages */
 
-#define PERROR_VERSION "2.10"
+#define PERROR_VERSION "2.11"
 
 #include <my_global.h>
 #include <my_sys.h>
 #include <m_string.h>
 #include <errno.h>
 #include <my_getopt.h>
-#ifdef HAVE_NDBCLUSTER_DB
-#include "../ndb/src/ndbapi/ndberror.c"
-#include "../ndb/src/kernel/error/ndbd_exit_codes.c"
+#ifdef WITH_NDBCLUSTER_STORAGE_ENGINE
+#include "../storage/ndb/src/ndbapi/ndberror.c"
+#include "../storage/ndb/src/kernel/error/ndbd_exit_codes.c"
+#include "../storage/ndb/include/mgmapi/mgmapi_error.h"
 #endif
 
 static my_bool verbose, print_all_codes;
 
-#ifdef HAVE_NDBCLUSTER_DB
+#include "../include/my_base.h"
+#include "../mysys/my_handler_errors.h"
+#include "../include/my_handler.h"
+
+#ifdef WITH_NDBCLUSTER_STORAGE_ENGINE
 static my_bool ndb_code;
 static char ndb_string[1024];
+int mgmapi_error_string(int err_no, char *str, int size)
+{
+  int i;
+  for (i= 0; i < ndb_mgm_noOfErrorMsgs; i++)
+  {
+    if ((int)ndb_mgm_error_msgs[i].code == err_no)
+    {
+      my_snprintf(str, size-1, "%s", ndb_mgm_error_msgs[i].msg);
+      str[size-1]= '\0';
+      return 0;
+    }
+  }
+  return -1;
+}
 #endif
 
 static struct my_option my_long_options[] =
@@ -41,19 +59,19 @@ static struct my_option my_long_options[] =
    NO_ARG, 0, 0, 0, 0, 0, 0},
   {"info", 'I', "Synonym for --help.",  0, 0, 0, GET_NO_ARG,
    NO_ARG, 0, 0, 0, 0, 0, 0},
-#ifdef HAVE_NDBCLUSTER_DB
-  {"ndb", 257, "Ndbcluster storage engine specific error codes.",  (gptr*) &ndb_code,
-   (gptr*) &ndb_code, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
+#ifdef WITH_NDBCLUSTER_STORAGE_ENGINE
+  {"ndb", 257, "Ndbcluster storage engine specific error codes.",  (uchar**) &ndb_code,
+   (uchar**) &ndb_code, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
 #endif
 #ifdef HAVE_SYS_ERRLIST
   {"all", 'a', "Print all the error messages and the number.",
-   (gptr*) &print_all_codes, (gptr*) &print_all_codes, 0, GET_BOOL, NO_ARG,
+   (uchar**) &print_all_codes, (uchar**) &print_all_codes, 0, GET_BOOL, NO_ARG,
    0, 0, 0, 0, 0, 0},
 #endif
   {"silent", 's', "Only print the error message.", 0, 0, 0, GET_NO_ARG, NO_ARG,
    0, 0, 0, 0, 0, 0},
-  {"verbose", 'v', "Print error code and message (default).", (gptr*) &verbose,
-   (gptr*) &verbose, 0, GET_BOOL, NO_ARG, 1, 0, 0, 0, 0, 0},
+  {"verbose", 'v', "Print error code and message (default).", (uchar**) &verbose,
+   (uchar**) &verbose, 0, GET_BOOL, NO_ARG, 1, 0, 0, 0, 0, 0},
   {"version", 'V', "Displays version information and exits.",
    0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
   {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
@@ -68,36 +86,6 @@ typedef struct ha_errors {
 
 static HA_ERRORS ha_errlist[]=
 {
-  { 120,"Didn't find key on read or update" },
-  { 121,"Duplicate key on write or update" },
-  { 123,"Someone has changed the row since it was read (while the table was locked to prevent it)" },
-  { 124,"Wrong index given to function" },
-  { 126,"Index file is crashed" },
-  { 127,"Record-file is crashed" },
-  { 128,"Out of memory" },
-  { 130,"Incorrect file format" },
-  { 131,"Command not supported by database" },
-  { 132,"Old database file" },
-  { 133,"No record read before update" },
-  { 134,"Record was already deleted (or record file crashed)" },
-  { 135,"No more room in record file" },
-  { 136,"No more room in index file" },
-  { 137,"No more records (read after end of file)" },
-  { 138,"Unsupported extension used for table" },
-  { 139,"Too big row"},
-  { 140,"Wrong create options"},
-  { 141,"Duplicate unique key or constraint on write or update"},
-  { 142,"Unknown character set used"},
-  { 143,"Conflicting table definitions in sub-tables of MERGE table"},
-  { 144,"Table is crashed and last repair failed"},
-  { 145,"Table was marked as crashed and should be repaired"},
-  { 146,"Lock timed out; Retry transaction"},
-  { 147,"Lock table is full;  Restart program with a larger locktable"},
-  { 148,"Updates are not allowed under a read only transactions"},
-  { 149,"Lock deadlock; Retry transaction"},
-  { 150,"Foreign key constraint is incorrectly formed"},
-  { 151,"Cannot add a child row"},
-  { 152,"Cannot delete a parent row"},
   { -30999, "DB_INCOMPLETE: Sync didn't finish"},
   { -30998, "DB_KEYEMPTY: Key/data deleted or never created"},
   { -30997, "DB_KEYEXIST: The key/data pair already exists"},
@@ -179,6 +167,17 @@ static const char *get_ha_error_msg(int code)
 {
   HA_ERRORS *ha_err_ptr;
 
+  /*
+    If you got compilation error here about compile_time_assert array, check
+    that every HA_ERR_xxx constant has a corresponding error message in
+    handler_error_messages[] list (check mysys/ma_handler_errors.h and
+    include/my_base.h).
+  */
+  compile_time_assert(HA_ERR_FIRST + array_elements(handler_error_messages) ==
+                      HA_ERR_LAST + 1);
+  if (code >= HA_ERR_FIRST && code <= HA_ERR_LAST)
+    return handler_error_messages[code - HA_ERR_FIRST];
+
   for (ha_err_ptr=ha_errlist ; ha_err_ptr->errcode ;ha_err_ptr++)
     if (ha_err_ptr->errcode == code)
       return ha_err_ptr->msg;
@@ -186,16 +185,43 @@ static const char *get_ha_error_msg(int code)
 }
 
 
+#if defined(__WIN__)
+static my_bool print_win_error_msg(DWORD error, my_bool verbose)
+{
+  LPTSTR s;
+  if (FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
+                    FORMAT_MESSAGE_FROM_SYSTEM,
+                    NULL, error, 0, (LPTSTR)&s, 0,
+                    NULL))
+  {
+    if (verbose)
+      printf("Win32 error code %d: %s", error, s);
+    else
+      puts(s);
+    LocalFree(s);
+    return 0;
+  }
+  return 1;
+}
+#endif
+
+
+
 int main(int argc,char *argv[])
 {
   int error,code,found;
   const char *msg;
   char *unknown_error = 0;
+#if defined(__WIN__)
+  my_bool skip_win_message= 0;
+#endif
   MY_INIT(argv[0]);
 
   if (get_options(&argc,&argv))
     exit(1);
 
+  my_handler_error_register();
+
   error=0;
 #ifdef HAVE_SYS_ERRLIST
   if (print_all_codes)
@@ -236,11 +262,12 @@ int main(int argc,char *argv[])
 
       found=0;
       code=atoi(*argv);
-#ifdef HAVE_NDBCLUSTER_DB
+#ifdef WITH_NDBCLUSTER_STORAGE_ENGINE
       if (ndb_code)
       {
-	if ((ndb_error_string(code, ndb_string, sizeof(ndb_string)) < 0) &&
-	    (ndbd_exit_string(code, ndb_string, sizeof(ndb_string)) < 0))
+        if ((ndb_error_string(code, ndb_string, sizeof(ndb_string)) < 0) &&
+            (ndbd_exit_string(code, ndb_string, sizeof(ndb_string)) < 0) &&
+            (mgmapi_error_string(code, ndb_string, sizeof(ndb_string)) < 0))
 	{
           msg= 0;
 	}
@@ -275,30 +302,36 @@ int main(int argc,char *argv[])
                        (const uchar*) "Unknown Error", 13) &&
           (!unknown_error || strcmp(msg, unknown_error)))
       {
-	found=1;
+	found= 1;
 	if (verbose)
-	  printf("OS error code %3d:  %s\n",code,msg);
+	  printf("OS error code %3d:  %s\n", code, msg);
 	else
 	  puts(msg);
       }
-
+      if ((msg= get_ha_error_msg(code)))
+      {
+        found= 1;
+        if (verbose)
+          printf("MySQL error code %3d: %s\n", code, msg);
+        else
+          puts(msg);
+      }
       if (!found)
       {
-        /* Error message still not found, look in handler error codes */
-        if (!(msg=get_ha_error_msg(code)))
+#if defined(__WIN__)
+        if (!(skip_win_message= !print_win_error_msg((DWORD)code, verbose)))
         {
-	  fprintf(stderr,"Illegal error code: %d\n",code);
-	  error=1;
-        }
-        else
-        {
-          found= 1;
-          if (verbose)
-            printf("MySQL error code %3d: %s\n",code,msg);
-          else
-            puts(msg);
+#endif
+          fprintf(stderr,"Illegal error code: %d\n",code);
+          error=1;
+#if defined(__WIN__)
         }
+#endif
       }
+#if defined(__WIN__)
+      if (!skip_win_message)
+        print_win_error_msg((DWORD)code, verbose);
+#endif
     }
   }
 
