@@ -1,5 +1,5 @@
 # -*- cperl -*-
-# Copyright (c) 2005, 2010, Oracle and/or its affiliates. All rights reserved.
+# Copyright (C) 2005-2006 MySQL AB
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,8 +22,11 @@ use File::Basename;
 use IO::File();
 use strict;
 
+use My::Config;
+
 sub collect_test_cases ($);
-sub collect_one_test_case ($$$$$$$);
+sub collect_one_suite ($);
+sub collect_one_test_case ($$$$$$$$$);
 
 sub mtr_options_from_test_file($$);
 
@@ -52,149 +55,44 @@ sub init_pattern {
 ##############################################################################
 
 sub collect_test_cases ($) {
-  my $suite= shift;             # Test suite name
-
-  my $testdir;
-  my $resdir;
-
   $do_test= init_pattern($::opt_do_test, "--do-test");
   $skip_test= init_pattern($::opt_skip_test, "--skip-test");
 
-  if ( $suite eq "main" )
+  my $suites= shift; # Semicolon separated list of test suites
+  my $cases = [];    # Array of hash
+
+  foreach my $suite (split(",", $suites))
   {
-    $testdir= "$::glob_mysql_test_dir/t";
-    $resdir=  "$::glob_mysql_test_dir/r";
-  }
-  else
-  {
-    $testdir= "$::glob_mysql_test_dir/suite/$suite/t";
-    $resdir=  "$::glob_mysql_test_dir/suite/$suite/r";
+    push(@$cases, collect_one_suite($suite));
   }
 
-  my $cases = [];           # Array of hash, will be array of C struct
-
-  opendir(TESTDIR, $testdir) or mtr_error("Can't open dir \"$testdir\": $!");
-
-  # ----------------------------------------------------------------------
-  # Disable some tests listed in disabled.def
-  # ----------------------------------------------------------------------
-  my %disabled;
-  if ( open(DISABLED, "$testdir/disabled.def" ) )
-  {
-    while ( <DISABLED> )
-      {
-        chomp;
-        if ( /^\s*(\S+)\s*:\s*(.*?)\s*$/ )
-          {
-            $disabled{$1}= $2;
-          }
-      }
-    close DISABLED;
-  }
 
   if ( @::opt_cases )
   {
-    foreach my $tname ( @::opt_cases )
+    # Check that the tests specified was found
+    # in at least one suite
+    foreach my $test_name_spec ( @::opt_cases )
     {
-      # Run in specified order, no sort
-      my $elem= undef;
-      my $component_id= undef;
-
-      # Get rid of directory part (path). Leave the extension since it is used
-      # to understand type of the test.
-
-      $tname = basename($tname);
-
-      # Check if the extenstion has been specified.
-
-      if ( mtr_match_extension($tname, "test") )
+      my $found= 0;
+      my ($sname, $tname, $extension)= split_testname($test_name_spec);
+      foreach my $test ( @$cases )
       {
-        $elem= $tname;
-        $tname=~ s/\.test$//;
-        $component_id= 'mysqld';
+	# test->{name} is always in suite.name format
+	if ( $test->{name} =~ /.*\.$tname/ )
+	{
+	  $found= 1;
+	}
       }
-      elsif ( mtr_match_extension($tname, "imtest") )
+      if ( not $found )
       {
-        $elem= $tname;
-        $tname =~ s/\.imtest$//;
-        $component_id= 'im';
+	mtr_error("Could not find $tname in any suite");
       }
-
-      # If target component is known, check that the specified test case
-      # exists.
-      #
-      # Otherwise, try to guess the target component.
-
-      if ( $component_id )
-      {
-        if ( ! -f "$testdir/$elem")
-        {
-          mtr_error("Test case $tname ($testdir/$elem) is not found");
-        }
-      }
-      else
-      {
-        my $mysqld_test_exists = -f "$testdir/$tname.test";
-        my $im_test_exists = -f "$testdir/$tname.imtest";
-
-        if ( $mysqld_test_exists and $im_test_exists )
-        {
-          mtr_error("Ambiguous test case name ($tname)");
-        }
-        elsif ( ! $mysqld_test_exists and ! $im_test_exists )
-        {
-          mtr_error("Test case $tname is not found");
-        }
-        elsif ( $mysqld_test_exists )
-        {
-          $elem= "$tname.test";
-          $component_id= 'mysqld';
-        }
-        elsif ( $im_test_exists )
-        {
-          $elem= "$tname.imtest";
-          $component_id= 'im';
-        }
-      }
-
-      collect_one_test_case($testdir,$resdir,$tname,$elem,$cases,\%disabled,
-        $component_id);
     }
-    closedir TESTDIR;
-  }
-  else
-  {
-    foreach my $elem ( sort readdir(TESTDIR) )
-    {
-      my $component_id= undef;
-      my $tname= undef;
-
-      if ($tname= mtr_match_extension($elem, 'test'))
-      {
-        $component_id = 'mysqld';
-      }
-      elsif ($tname= mtr_match_extension($elem, 'imtest'))
-      {
-        $component_id = 'im';
-      }
-      else
-      {
-        next;
-      }
-
-      # Skip tests that does not match the --do-test= filter
-      next if ($do_test and not $tname =~ /$do_test/o);
-
-      collect_one_test_case($testdir,$resdir,$tname,$elem,$cases,\%disabled,
-        $component_id);
-    }
-    closedir TESTDIR;
   }
 
-  # Reorder the test cases in an order that will make them faster to run
   if ( $::opt_reorder )
   {
-
+    # Reorder the test cases in an order that will make them faster to run
     my %sort_criteria;
 
     # Make a mapping of test name to a string that represents how that test
@@ -246,10 +144,8 @@ sub collect_test_cases ($) {
 	push(@criteria, "ndb=" . ($tinfo->{'ndb_test'} ? "1" : "0"));
 	# Group test with equal options together.
 	# Ending with "~" makes empty sort later than filled
-	if ( $tinfo->{'master_opt'}[0] )
-	{
-	  push(@criteria, join("!", sort @{$tinfo->{'master_opt'}}) . "~");
-	}
+	push(@criteria, join("!", sort @{$tinfo->{'master_opt'}}) . "~");
+
 	$sort_criteria{$test_name} = join(" ", @criteria);
       }
     }
@@ -269,6 +165,330 @@ sub collect_test_cases ($) {
   }
 
   return $cases;
+
+}
+
+# Valid extensions and their corresonding component id
+my %exts = ( 'test' => 'mysqld',
+	     'imtest' => 'im'
+	   );
+
+
+# Returns (suitename, testname, extension)
+sub split_testname {
+  my ($test_name)= @_;
+
+  # Get rid of directory part and split name on .'s
+  my @parts= split(/\./, basename($test_name));
+
+  if (@parts == 1){
+    # Only testname given, ex: alias
+    return (undef , $parts[0], undef);
+  } elsif (@parts == 2) {
+    # Either testname.test or suite.testname given
+    # Ex. main.alias or alias.test
+
+    if (defined $exts{$parts[1]})
+    {
+      return (undef , $parts[0], $parts[1]);
+    }
+    else
+    {
+      return ($parts[0], $parts[1], undef);
+    }
+
+  } elsif (@parts == 3) {
+    # Fully specified suitename.testname.test
+    # ex main.alias.test
+    return ( $parts[0], $parts[1], $parts[2]);
+  }
+
+  mtr_error("Illegal format of test name: $test_name");
+}
+
+
+sub collect_one_suite($)
+{
+  my $suite= shift;  # Test suite name
+  my @cases;  # Array of hash
+
+  mtr_verbose("Collecting: $suite");
+
+  my $suitedir= "$::glob_mysql_test_dir"; # Default
+  if ( $suite ne "main" )
+  {
+    $suitedir= mtr_path_exists("$suitedir/suite/$suite",
+			       "$suitedir/$suite");
+    mtr_verbose("suitedir: $suitedir");
+  }
+
+  my $testdir= "$suitedir/t";
+  my $resdir=  "$suitedir/r";
+
+  # ----------------------------------------------------------------------
+  # Build a hash of disabled testcases for this suite
+  # ----------------------------------------------------------------------
+  my %disabled;
+  if ( open(DISABLED, "$testdir/disabled.def" ) )
+  {
+    while ( <DISABLED> )
+      {
+        chomp;
+        if ( /^\s*(\S+)\s*:\s*(.*?)\s*$/ )
+          {
+            $disabled{$1}= $2;
+          }
+      }
+    close DISABLED;
+  }
+
+  # Read suite.opt file
+  my $suite_opt_file=  "$testdir/suite.opt";
+  my $suite_opts= [];
+  if ( -f $suite_opt_file )
+  {
+    $suite_opts= mtr_get_opts_from_file($suite_opt_file);
+  }
+
+  if ( @::opt_cases )
+  {
+    # Collect in specified order
+    foreach my $test_name_spec ( @::opt_cases )
+    {
+      my ($sname, $tname, $extension)= split_testname($test_name_spec);
+
+      # The test name parts have now been defined
+      #print "  suite_name: $sname\n";
+      #print "  tname:      $tname\n";
+      #print "  extension:  $extension\n";
+
+      # Check cirrect suite if suitename is defined
+      next if (defined $sname and $suite ne $sname);
+
+      my $component_id;
+      if ( defined $extension )
+      {
+	my $full_name= "$testdir/$tname.$extension";
+	# Extension was specified, check if the test exists
+        if ( ! -f $full_name)
+        {
+	  # This is only an error if suite was specified, otherwise it
+	  # could exist in another suite
+          mtr_error("Test '$full_name' was not found in suite '$sname'")
+	    if $sname;
+
+	  next;
+        }
+	$component_id= $exts{$extension};
+      }
+      else
+      {
+	# No extension was specified
+	my ($ext, $component);
+	while (($ext, $component)= each %exts) {
+	  my $full_name= "$testdir/$tname.$ext";
+
+	  if ( ! -f $full_name ) {
+	    next;
+	  }
+	  $component_id= $component;
+	  $extension= $ext;
+	}
+	# Test not found here, could exist in other suite
+	next unless $component_id;
+      }
+
+      collect_one_test_case($testdir,$resdir,$suite,$tname,
+                            "$tname.$extension",\@cases,\%disabled,
+			    $component_id,$suite_opts);
+    }
+  }
+  else
+  {
+    opendir(TESTDIR, $testdir) or mtr_error("Can't open dir \"$testdir\": $!");
+
+    foreach my $elem ( sort readdir(TESTDIR) )
+    {
+      my $component_id= undef;
+      my $tname= undef;
+
+      if ($tname= mtr_match_extension($elem, 'test'))
+      {
+        $component_id = 'mysqld';
+      }
+      elsif ($tname= mtr_match_extension($elem, 'imtest'))
+      {
+        $component_id = 'im';
+      }
+      else
+      {
+        next;
+      }
+
+      # Skip tests that does not match the --do-test= filter
+      next if ($do_test and not $tname =~ /$do_test/o);
+
+      collect_one_test_case($testdir,$resdir,$suite,$tname,
+                            $elem,\@cases,\%disabled,$component_id,
+			    $suite_opts);
+    }
+    closedir TESTDIR;
+  }
+
+
+  #  Return empty list if no testcases found
+  return if (@cases == 0);
+
+  # ----------------------------------------------------------------------
+  # Read combinations for this suite and build testcases x combinations
+  # if any combinations exists
+  # ----------------------------------------------------------------------
+  if ( ! $::opt_skip_combination )
+  {
+    my @combinations;
+    my $combination_file= "$suitedir/combinations";
+    #print "combination_file: $combination_file\n";
+    if (@::opt_combinations)
+    {
+      # take the combination from command-line
+      mtr_verbose("Take the combination from command line");
+      foreach my $combination (@::opt_combinations) {
+	my $comb= {};
+	$comb->{name}= $combination;
+	push(@{$comb->{comb_opt}}, $combination);
+	push(@combinations, $comb);
+      }
+    }
+    elsif (-f $combination_file )
+    {
+      # Read combinations file in my.cnf format
+      mtr_verbose("Read combinations file");
+      my $config= My::Config->new($combination_file);
+
+      foreach my $group ($config->groups()) {
+	my $comb= {};
+	$comb->{name}= $group->name();
+        foreach my $option ( $group->options() ) {
+	  push(@{$comb->{comb_opt}}, $option->name()."=".$option->value());
+	}
+	push(@combinations, $comb);
+      }
+    }
+
+    if (@combinations)
+    {
+      print " - adding combinations\n";
+      #print_testcases(@cases);
+
+      my @new_cases;
+      foreach my $comb (@combinations)
+      {
+	foreach my $test (@cases)
+	{
+	  #print $test->{name}, " ", $comb, "\n";
+	  my $new_test= {};
+
+	  while (my ($key, $value) = each(%$test)) {
+	    if (ref $value eq "ARRAY") {
+	      push(@{$new_test->{$key}}, @$value);
+	    } else {
+	      $new_test->{$key}= $value;
+	    }
+	  }
+
+	  # Append the combination options to master_opt and slave_opt
+	  push(@{$new_test->{master_opt}}, @{$comb->{comb_opt}});
+	  push(@{$new_test->{slave_opt}}, @{$comb->{comb_opt}});
+
+	  # Add combination name shrt name
+	  $new_test->{combination}= $comb->{name};
+
+	  # Add the new test to new test cases list
+	  push(@new_cases, $new_test);
+	}
+      }
+      #print_testcases(@new_cases);
+      @cases= @new_cases;
+      #print_testcases(@cases);
+    }
+  }
+
+  optimize_cases(\@cases);
+  #print_testcases(@cases);
+
+  return @cases;
+}
+
+
+#
+# Loop through all test cases
+# - optimize which test to run by skipping unnecessary ones
+# - update settings if necessary
+#
+sub optimize_cases {
+  my ($cases)= @_;
+
+  foreach my $tinfo ( @$cases )
+  {
+    # Skip processing if already marked as skipped
+    next if $tinfo->{skip};
+
+    # =======================================================
+    # If a special binlog format was selected with
+    # --mysqld=--binlog-format=x, skip all test that does not
+    # support it
+    # =======================================================
+    #print "used_binlog_format: $::used_binlog_format\n";
+    if (defined $::used_binlog_format )
+    {
+      # =======================================================
+      # Fixed --binlog-format=x specified on command line
+      # =======================================================
+      if ( defined $tinfo->{'binlog_formats'} )
+      {
+	#print "binlog_formats: ". join(", ", @{$tinfo->{binlog_formats}})."\n";
+
+	# The test supports different binlog formats
+	# check if the selected one is ok
+	my $supported=
+	  grep { $_ eq $::used_binlog_format } @{$tinfo->{'binlog_formats'}};
+	if ( !$supported )
+	{
+	  $tinfo->{'skip'}= 1;
+	  $tinfo->{'comment'}=
+	    "Doesn't support --binlog-format='$::used_binlog_format'";
+	}
+      }
+    }
+    else
+    {
+      # =======================================================
+      # Use dynamic switching of binlog format
+      # =======================================================
+
+      # Get binlog-format used by this test from master_opt
+      my $test_binlog_format;
+      foreach my $opt ( @{$tinfo->{master_opt}} ) {
+	$test_binlog_format=
+	  mtr_match_prefix($opt, "--binlog-format=") || $test_binlog_format;
+      }
+
+      if (defined $test_binlog_format and
+	  defined $tinfo->{binlog_formats} )
+      {
+	my $supported=
+	  grep { $_ eq $test_binlog_format } @{$tinfo->{'binlog_formats'}};
+	if ( !$supported )
+	{
+	  $tinfo->{'skip'}= 1;
+	  $tinfo->{'comment'}=
+	    "Doesn't support --binlog-format='$test_binlog_format'";
+	  next;
+	}
+      }
+    }
+
+  }
 }
 
 
@@ -279,14 +499,16 @@ sub collect_test_cases ($) {
 ##############################################################################
 
 
-sub collect_one_test_case($$$$$$$) {
+sub collect_one_test_case($$$$$$$$$) {
   my $testdir= shift;
   my $resdir=  shift;
+  my $suite=   shift;
   my $tname=   shift;
   my $elem=    shift;
   my $cases=   shift;
   my $disabled=shift;
   my $component_id= shift;
+  my $suite_opts= shift;
 
   my $path= "$testdir/$elem";
 
@@ -301,7 +523,7 @@ sub collect_one_test_case($$$$$$$) {
 
 
   my $tinfo= {};
-  $tinfo->{'name'}= $tname;
+  $tinfo->{'name'}= basename($suite) . ".$tname";
   $tinfo->{'result_file'}= "$resdir/$tname.result";
   $tinfo->{'component_id'} = $component_id;
   push(@$cases, $tinfo);
@@ -356,6 +578,16 @@ sub collect_one_test_case($$$$$$$) {
   $tinfo->{'slave_opt'}=  [];
   $tinfo->{'slave_mi'}=   [];
 
+
+  # Add suite opts
+  foreach my $opt ( @$suite_opts )
+  {
+    mtr_verbose($opt);
+    push(@{$tinfo->{'master_opt'}}, $opt);
+    push(@{$tinfo->{'slave_opt'}}, $opt);
+  }
+
+  # Add master opts
   if ( -f $master_opt_file )
   {
 
@@ -416,6 +648,7 @@ sub collect_one_test_case($$$$$$$) {
     }
   }
 
+  # Add slave opts
   if ( -f $slave_opt_file )
   {
     my $slave_opt= mtr_get_opts_from_file($slave_opt_file);
@@ -538,6 +771,13 @@ sub collect_one_test_case($$$$$$$) {
 	if ( $::used_default_engine =~ /^innodb/i );
     }
 
+    #enable federated for this test
+    if ($tinfo->{'federated_test'})
+    {
+      push(@{$tinfo->{'master_opt'}}, "--loose-federated");
+      push(@{$tinfo->{'slave_opt'}}, "--loose-federated");
+    }
+
     if ( $tinfo->{'big_test'} and ! $::opt_big_test )
     {
       $tinfo->{'skip'}= 1;
@@ -559,14 +799,6 @@ sub collect_one_test_case($$$$$$$) {
       return;
     }
 
-    if ( defined $tinfo->{'binlog_format'} and
-	 ! ( $tinfo->{'binlog_format'} eq $::used_binlog_format ) )
-    {
-      $tinfo->{'skip'}= 1;
-      $tinfo->{'comment'}= "Not running with binlog format '$tinfo->{'binlog_format'}'";
-      return;
-    }
-
     if ( $tinfo->{'need_debug'} && ! $::debug_compiled_binaries )
     {
       $tinfo->{'skip'}= 1;
@@ -581,7 +813,7 @@ sub collect_one_test_case($$$$$$$) {
       {
 	# Ndb is not supported, skip it
 	$tinfo->{'skip'}= 1;
-	$tinfo->{'comment'}= "No ndbcluster support or ndb tests disabled";
+	$tinfo->{'comment'}= "No ndbcluster support";
 	return;
       }
       elsif ( $::opt_skip_ndbcluster )
@@ -609,7 +841,7 @@ sub collect_one_test_case($$$$$$$) {
     if ( $tinfo->{'innodb_test'} )
     {
       # This is a test that need innodb
-      if ( $::mysqld_variables{'innodb'} eq "FALSE" )
+      if ( $::mysqld_variables{'innodb'} ne "TRUE" )
       {
 	# innodb is not supported, skip it
 	$tinfo->{'skip'}= 1;
@@ -645,16 +877,28 @@ sub collect_one_test_case($$$$$$$) {
 # the specified value in "tinfo"
 our @tags=
 (
+
+ ["include/have_binlog_format_row.inc", "binlog_formats", ["row"]],
+ ["include/have_binlog_format_statement.inc", "binlog_formats", ["statement"]],
+ ["include/have_binlog_format_mixed.inc", "binlog_formats", ["mixed"]],
+ ["include/have_binlog_format_mixed_or_row.inc",
+  "binlog_formats", ["mixed", "row"]],
+ ["include/have_binlog_format_mixed_or_statement.inc",
+  "binlog_formats", ["mixed", "statement"]],
+ ["include/have_binlog_format_row_or_statement.inc",
+  "binlog_formats", ["row", "statement"]],
+
  ["include/have_innodb.inc", "innodb_test", 1],
- ["include/have_binlog_format_row.inc", "binlog_format", "row"],
- ["include/have_binlog_format_statement.inc", "binlog_format", "stmt"],
- ["include/have_binlog_format_mixed.inc", "binlog_format", "mixed"],
  ["include/have_log_bin.inc", "need_binlog", 1],
  ["include/big_test.inc", "big_test", 1],
  ["include/have_debug.inc", "need_debug", 1],
  ["include/have_ndb.inc", "ndb_test", 1],
+ ["include/have_multi_ndb.inc", "ndb_test", 1],
  ["include/have_ndb_extra.inc", "ndb_extra", 1],
+ ["include/ndb_master-slave.inc", "ndb_test", 1],
  ["require_manager", "require_manager", 1],
+ ["include/federated.inc", "federated_test", 1],
+ ["include/have_federated_db.inc", "federated_test", 1],
 );
 
 sub mtr_options_from_test_file($$) {
@@ -674,8 +918,8 @@ sub mtr_options_from_test_file($$) {
     {
       if ( index($line, $tag->[0]) >= 0 )
       {
-	# Tag matched, assign value to "tinfo"
-	$tinfo->{"$tag->[1]"}= $tag->[2];
+    	  # Tag matched, assign value to "tinfo"
+        $tinfo->{"$tag->[1]"}= $tag->[2];
       }
     }
 
@@ -696,8 +940,29 @@ sub mtr_options_from_test_file($$) {
 	mtr_options_from_test_file($tinfo, $sourced_file);
       }
     }
-
   }
 }
 
+
+sub print_testcases {
+  my (@cases)= @_;
+
+  print "=" x 60, "\n";
+  foreach my $test (@cases){
+    print "[", $test->{name}, "]", "\n";
+    while ((my ($key, $value)) = each(%$test)) {
+      print " ", $key, "=";
+      if (ref $value eq "ARRAY") {
+	print join(", ", @$value);
+      } else {
+	print $value;
+      }
+      print "\n";
+    }
+    print "\n";
+  }
+  print "=" x 60, "\n";
+}
+
+
 1;
