@@ -1,9 +1,8 @@
-/* Copyright (C) 2000 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
+/* Copyright (C) 2002-2006 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,6 +19,10 @@
 #include <my_sys.h>
 #include <mysys_err.h>
 #include <my_getopt.h>
+#include <errno.h>
+
+typedef void (*init_func_p)(const struct my_option *option, uchar* *variable,
+                            longlong value);
 
 static void default_reporter(enum loglevel level, const char *format, ...);
 my_error_reporter my_getopt_error_reporter= &default_reporter;
@@ -33,8 +36,14 @@ my_bool getopt_compare_strings(const char *s,
 static longlong getopt_ll(char *arg, const struct my_option *optp, int *err);
 static ulonglong getopt_ull(char *arg, const struct my_option *optp,
 			    int *err);
-static void init_variables(const struct my_option *options);
-static int setval(const struct my_option *opts, gptr *value, char *argument,
+static double getopt_double(char *arg, const struct my_option *optp, int *err);
+static void init_variables(const struct my_option *options,
+                           init_func_p init_one_value);
+static void init_one_value(const struct my_option *option, uchar* *variable,
+			   longlong value);
+static void fini_one_value(const struct my_option *option, uchar* *variable,
+			   longlong value);
+static int setval(const struct my_option *opts, uchar* *value, char *argument,
 		  my_bool set_maximum_value);
 static char *check_struct_option(char *cur_arg, char *key_name);
 
@@ -59,6 +68,13 @@ char *disabled_my_option= (char*) "0";
 
 my_bool my_getopt_print_errors= 1;
 
+/* 
+   This is a flag that can be set in client programs. 1 means that
+   my_getopt will skip over options it does not know how to handle.
+*/
+
+my_bool my_getopt_skip_unknown= 0;
+
 static void default_reporter(enum loglevel level,
                              const char *format, ...)
 {
@@ -70,6 +86,7 @@ static void default_reporter(enum loglevel level,
     fprintf(stderr, "%s", "Info: ");
   vfprintf(stderr, format, args);
   va_end(args);
+  fputc('\n', stderr);
   fflush(stderr);
 }
 
@@ -83,9 +100,9 @@ static void default_reporter(enum loglevel level,
   one. Call function 'get_one_option()' once for each option.
 */
 
-static gptr* (*getopt_get_addr)(const char *, uint, const struct my_option *);
+static uchar** (*getopt_get_addr)(const char *, uint, const struct my_option *);
 
-void my_getopt_register_get_addr(gptr* (*func_addr)(const char *, uint,
+void my_getopt_register_get_addr(uchar** (*func_addr)(const char *, uint,
 						    const struct my_option *))
 {
   getopt_get_addr= func_addr;
@@ -95,22 +112,26 @@ int handle_options(int *argc, char ***argv,
 		   const struct my_option *longopts,
                    my_get_one_option get_one_option)
 {
-  uint opt_found, argvpos= 0, length, i;
+  uint opt_found, argvpos= 0, length;
   my_bool end_of_options= 0, must_be_var, set_maximum_value,
           option_is_loose;
   char **pos, **pos_end, *optend, *prev_found,
        *opt_str, key_name[FN_REFLEN];
   const struct my_option *optp;
-  gptr *value;
-  int error;
+  uchar* *value;
+  int error, i;
 
   LINT_INIT(opt_found);
+  /* handle_options() assumes arg0 (program name) always exists */
+  DBUG_ASSERT(argc && *argc >= 1);
+  DBUG_ASSERT(argv && *argv);
   (*argc)--; /* Skip the program name */
   (*argv)++; /*      --- || ----      */
-  init_variables(longopts);
+  init_variables(longopts, init_one_value);
 
   for (pos= *argv, pos_end=pos+ *argc; pos != pos_end ; pos++)
   {
+    char **first= pos;
     char *cur_arg= *pos;
     if (cur_arg[0] == '-' && cur_arg[1] && !end_of_options) /* must be opt */
     {
@@ -133,7 +154,7 @@ int handle_options(int *argc, char ***argv,
 	    {
 	      if (my_getopt_print_errors)
                 my_getopt_error_reporter(ERROR_LEVEL,
-                                         "%s: Option '-O' requires an argument\n",
+                                         "%s: Option '-O' requires an argument",
                                          my_progname);
 	      return EXIT_ARGUMENT_REQUIRED;
 	    }
@@ -151,7 +172,7 @@ int handle_options(int *argc, char ***argv,
 	    {
 	      if (my_getopt_print_errors)
                 my_getopt_error_reporter(ERROR_LEVEL,
-                                         "%s: Option '--set-variable' requires an argument\n",
+                                         "%s: Option '--set-variable' requires an argument",
                                          my_progname);
 	      return EXIT_ARGUMENT_REQUIRED;
 	    }
@@ -165,7 +186,7 @@ int handle_options(int *argc, char ***argv,
 	    {
 	      if (my_getopt_print_errors)
                 my_getopt_error_reporter(ERROR_LEVEL,
-                                         "%s: Option '--set-variable' requires an argument\n",
+                                         "%s: Option '--set-variable' requires an argument",
                                          my_progname);
 	      return EXIT_ARGUMENT_REQUIRED;
 	    }
@@ -195,6 +216,7 @@ int handle_options(int *argc, char ***argv,
 	  Find first the right option. Return error in case of an ambiguous,
 	  or unknown option
 	*/
+        LINT_INIT(prev_found);
 	optp= longopts;
 	if (!(opt_found= findopt(opt_str, length, &optp, &prev_found)))
 	{
@@ -216,18 +238,17 @@ int handle_options(int *argc, char ***argv,
 		/*
 		  We were called with a special prefix, we can reuse opt_found
 		*/
-		opt_str+= (special_opt_prefix_lengths[i] + 1);
+		opt_str+= special_opt_prefix_lengths[i] + 1;
+                length-= special_opt_prefix_lengths[i] + 1;
 		if (i == OPT_LOOSE)
 		  option_is_loose= 1;
-		if ((opt_found= findopt(opt_str, length -
-					(special_opt_prefix_lengths[i] + 1),
-					&optp, &prev_found)))
+		if ((opt_found= findopt(opt_str, length, &optp, &prev_found)))
 		{
 		  if (opt_found > 1)
 		  {
 		    if (my_getopt_print_errors)
                       my_getopt_error_reporter(ERROR_LEVEL,
-                                               "%s: ambiguous option '--%s-%s' (--%s-%s)\n",
+                                               "%s: ambiguous option '--%s-%s' (--%s-%s)",
                                                my_progname, special_opt_prefix[i],
                                                cur_arg, special_opt_prefix[i],
                                                prev_found);
@@ -245,7 +266,7 @@ int handle_options(int *argc, char ***argv,
 		    break;
 		  case OPT_ENABLE:
 		    optend= (optend && *optend == '0' && !(*(optend + 1))) ?
- 		      disabled_my_option : (char*) "1";
+                      disabled_my_option : (char*) "1";
 		    break;
 		  case OPT_MAXIMUM:
 		    set_maximum_value= 1;
@@ -254,17 +275,31 @@ int handle_options(int *argc, char ***argv,
 		  }
 		  break; /* break from the inner loop, main loop continues */
 		}
+                i= -1; /* restart the loop */
 	      }
 	    }
 	  }
 	  if (!opt_found)
 	  {
+            if (my_getopt_skip_unknown)
+            {
+              /*
+                preserve all the components of this unknown option, this may
+                occurr when the user provides options like: "-O foo" or
+                "--set-variable foo" (note that theres a space in there)
+                Generally, these kind of options are to be avoided
+              */
+              do {
+                (*argv)[argvpos++]= *first++;
+              } while (first <= pos);
+              continue;
+            }
 	    if (must_be_var)
 	    {
 	      if (my_getopt_print_errors)
                 my_getopt_error_reporter(option_is_loose ? 
                                            WARNING_LEVEL : ERROR_LEVEL,
-                                         "%s: unknown variable '%s'\n",
+                                         "%s: unknown variable '%s'",
                                          my_progname, cur_arg);
 	      if (!option_is_loose)
 		return EXIT_UNKNOWN_VARIABLE;
@@ -274,7 +309,7 @@ int handle_options(int *argc, char ***argv,
 	      if (my_getopt_print_errors)
                 my_getopt_error_reporter(option_is_loose ? 
                                            WARNING_LEVEL : ERROR_LEVEL,
-                                         "%s: unknown option '--%s'\n", 
+                                         "%s: unknown option '--%s'", 
                                          my_progname, cur_arg);
 	      if (!option_is_loose)
 		return EXIT_UNKNOWN_OPTION;
@@ -292,7 +327,7 @@ int handle_options(int *argc, char ***argv,
 	  {
 	    if (my_getopt_print_errors)
               my_getopt_error_reporter(ERROR_LEVEL,
-                                       "%s: variable prefix '%s' is not unique\n",
+                                       "%s: variable prefix '%s' is not unique",
                                        my_progname, opt_str);
 	    return EXIT_VAR_PREFIX_NOT_UNIQUE;
 	  }
@@ -300,7 +335,7 @@ int handle_options(int *argc, char ***argv,
 	  {
 	    if (my_getopt_print_errors)
               my_getopt_error_reporter(ERROR_LEVEL,
-                                       "%s: ambiguous option '--%s' (%s, %s)\n",
+                                       "%s: ambiguous option '--%s' (%s, %s)",
                                        my_progname, opt_str, prev_found, 
                                        optp->name);
 	    return EXIT_AMBIGUOUS_OPTION;
@@ -323,7 +358,7 @@ int handle_options(int *argc, char ***argv,
 	{
 	  if (my_getopt_print_errors)
             my_getopt_error_reporter(ERROR_LEVEL, 
-                                     "%s: option '%s' cannot take an argument\n",
+                                     "%s: option '%s' cannot take an argument",
                                      my_progname, optp->name);
 	  return EXIT_NO_ARGUMENT_ALLOWED;
 	}
@@ -336,7 +371,7 @@ int handle_options(int *argc, char ***argv,
 	  {
 	    if (my_getopt_print_errors)
               my_getopt_error_reporter(ERROR_LEVEL,
-                                       "%s: option '--%s' cannot take an argument\n",
+                                       "%s: option '--%s' cannot take an argument",
                                        my_progname, optp->name);
 	    return EXIT_NO_ARGUMENT_ALLOWED;
 	  }
@@ -358,13 +393,14 @@ int handle_options(int *argc, char ***argv,
 	    {
 	      my_getopt_error_reporter(WARNING_LEVEL,
 				       "%s: ignoring option '--%s' due to \
-invalid value '%s'\n",
+invalid value '%s'",
 				       my_progname, optp->name, optend);
 	      continue;
 	    }
-	    get_one_option(optp->id, optp,
-			   *((my_bool*) value) ?
-			   (char*) "1" : disabled_my_option);
+	    if (get_one_option(optp->id, optp,
+                               *((my_bool*) value) ?
+                               (char*) "1" : disabled_my_option))
+              return EXIT_ARGUMENT_INVALID;
 	    continue;
 	  }
 	  argument= optend;
@@ -389,7 +425,7 @@ invalid value '%s'\n",
 	  {
 	    if (my_getopt_print_errors)
               my_getopt_error_reporter(ERROR_LEVEL,
-                                       "%s: option '--%s' requires an argument\n",
+                                       "%s: option '--%s' requires an argument",
                                        my_progname, optp->name);
 	    return EXIT_ARGUMENT_REQUIRED;
 	  }
@@ -422,7 +458,8 @@ invalid value '%s'\n",
 		  optp->arg_type == NO_ARG)
 	      {
 		*((my_bool*) optp->value)= (my_bool) 1;
-		get_one_option(optp->id, optp, argument);
+		if (get_one_option(optp->id, optp, argument))
+                  return EXIT_UNSPECIFIED_ERROR;
 		continue;
 	      }
 	      else if (optp->arg_type == REQUIRED_ARG ||
@@ -441,7 +478,8 @@ invalid value '%s'\n",
                   {
                     if (optp->var_type == GET_BOOL)
                       *((my_bool*) optp->value)= (my_bool) 1;
-                    get_one_option(optp->id, optp, argument);
+                    if (get_one_option(optp->id, optp, argument))
+                      return EXIT_UNSPECIFIED_ERROR;
                     continue;
                   }
 		  /* Check if there are more arguments after this one */
@@ -449,7 +487,7 @@ invalid value '%s'\n",
 		  {
                     if (my_getopt_print_errors)
                       my_getopt_error_reporter(ERROR_LEVEL,
-                                               "%s: option '-%c' requires an argument\n",
+                                               "%s: option '-%c' requires an argument",
                                                my_progname, optp->id);
                     return EXIT_ARGUMENT_REQUIRED;
 		  }
@@ -462,11 +500,12 @@ invalid value '%s'\n",
 				 set_maximum_value)))
 	      {
                 my_getopt_error_reporter(ERROR_LEVEL,
-                                         "%s: Error while setting value '%s' to '%s'\n",
+                                         "%s: Error while setting value '%s' to '%s'",
                                          my_progname, argument, optp->name);
 		return error;
 	      }
-	      get_one_option(optp->id, optp, argument);
+	      if (get_one_option(optp->id, optp, argument))
+                return EXIT_UNSPECIFIED_ERROR;
 	      break;
 	    }
 	  }
@@ -474,7 +513,7 @@ invalid value '%s'\n",
 	  {
 	    if (my_getopt_print_errors)
               my_getopt_error_reporter(ERROR_LEVEL,
-                                       "%s: unknown option '-%c'\n", 
+                                       "%s: unknown option '-%c'", 
                                        my_progname, *optend);
 	    return EXIT_UNKNOWN_OPTION;
 	  }
@@ -485,13 +524,14 @@ invalid value '%s'\n",
       if ((error= setval(optp, value, argument, set_maximum_value)))
       {
         my_getopt_error_reporter(ERROR_LEVEL,
-                                 "%s: Error while setting value '%s' to '%s'\n",
+                                 "%s: Error while setting value '%s' to '%s'",
                                  my_progname, argument, optp->name);
 	return error;
       }
-      get_one_option(optp->id, optp, argument);
+      if (get_one_option(optp->id, optp, argument))
+        return EXIT_UNSPECIFIED_ERROR;
 
-      (*argc)--; /* option handled (shortorlong), decrease argument count */
+      (*argc)--; /* option handled (short or long), decrease argument count */
     }
     else /* non-option found */
       (*argv)[argvpos++]= cur_arg;
@@ -556,14 +596,14 @@ static char *check_struct_option(char *cur_arg, char *key_name)
   Will set the option value to given value
 */
 
-static int setval(const struct my_option *opts, gptr *value, char *argument,
+static int setval(const struct my_option *opts, uchar* *value, char *argument,
 		  my_bool set_maximum_value)
 {
   int err= 0;
 
   if (value && argument)
   {
-    gptr *result_pos= ((set_maximum_value) ?
+    uchar* *result_pos= ((set_maximum_value) ?
 		       opts->u_max_value : value);
 
     if (!result_pos)
@@ -574,19 +614,26 @@ static int setval(const struct my_option *opts, gptr *value, char *argument,
       *((my_bool*) result_pos)= (my_bool) atoi(argument) != 0;
       break;
     case GET_INT:
-    case GET_UINT:           /* fall through */
       *((int*) result_pos)= (int) getopt_ll(argument, opts, &err);
       break;
+    case GET_UINT:
+      *((uint*) result_pos)= (uint) getopt_ull(argument, opts, &err);
+      break;
     case GET_LONG:
-    case GET_ULONG:          /* fall through */
       *((long*) result_pos)= (long) getopt_ll(argument, opts, &err);
       break;
+    case GET_ULONG:
+      *((long*) result_pos)= (long) getopt_ull(argument, opts, &err);
+      break;
     case GET_LL:
       *((longlong*) result_pos)= getopt_ll(argument, opts, &err);
       break;
     case GET_ULL:
       *((ulonglong*) result_pos)= getopt_ull(argument, opts, &err);
       break;
+    case GET_DOUBLE:
+      *((double*) result_pos)= getopt_double(argument, opts, &err);
+      break;
     case GET_STR:
       *((char**) result_pos)= argument;
       break;
@@ -596,6 +643,15 @@ static int setval(const struct my_option *opts, gptr *value, char *argument,
       if (!(*((char**) result_pos)= my_strdup(argument, MYF(MY_WME))))
 	return EXIT_OUT_OF_MEMORY;
       break;
+    case GET_ENUM:
+      if (((*(int*)result_pos)= find_type(argument, opts->typelib, 2) - 1) < 0)
+        return EXIT_ARGUMENT_INVALID;
+      break;
+    case GET_SET:
+      *((ulonglong*)result_pos)= find_typeset(argument, opts->typelib, &err);
+      if (err)
+        return EXIT_ARGUMENT_INVALID;
+      break;
     default:    /* dummy default to avoid compiler warnings */
       break;
     }
@@ -687,13 +743,21 @@ my_bool getopt_compare_strings(register const char *s, register const char *t,
   be k|K for kilo, m|M for mega or g|G for giga.
 */
 
-static longlong eval_num_suffix (char *argument, int *error, char *option_name)
+static longlong eval_num_suffix(char *argument, int *error, char *option_name)
 {
   char *endchar;
   longlong num;
   
   *error= 0;
+  errno= 0;
   num= strtoll(argument, &endchar, 10);
+  if (errno == ERANGE)
+  {
+    my_getopt_error_reporter(ERROR_LEVEL,
+                             "Incorrect integer value: '%s'", argument);
+    *error= 1;
+    return 0;
+  }
   if (*endchar == 'k' || *endchar == 'K')
     num*= 1024L;
   else if (*endchar == 'm' || *endchar == 'M')
@@ -724,16 +788,70 @@ static longlong eval_num_suffix (char *argument, int *error, char *option_name)
 
 static longlong getopt_ll(char *arg, const struct my_option *optp, int *err)
 {
-  longlong num;
+  longlong num=eval_num_suffix(arg, err, (char*) optp->name);
+  return getopt_ll_limit_value(num, optp, NULL);
+}
+
+/*
+  function: getopt_ll_limit_value
+
+  Applies min/max/block_size to a numeric value of an option.
+  Returns "fixed" value.
+*/
+
+longlong getopt_ll_limit_value(longlong num, const struct my_option *optp,
+                               my_bool *fix)
+{
+  longlong old= num;
+  my_bool adjusted= FALSE;
+  char buf1[255], buf2[255];
   ulonglong block_size= (optp->block_size ? (ulonglong) optp->block_size : 1L);
-  
-  num= eval_num_suffix(arg, err, (char*) optp->name);
-  if (num > 0 && (ulonglong) num > (ulonglong) optp->max_value &&
+
+  if (num > 0 && ((ulonglong) num > (ulonglong) optp->max_value) &&
       optp->max_value) /* if max value is not set -> no upper limit */
+  {
     num= (ulonglong) optp->max_value;
+    adjusted= TRUE;
+  }
+
+  switch ((optp->var_type & GET_TYPE_MASK)) {
+  case GET_INT:
+    if (num > (longlong) INT_MAX)
+    {
+      num= ((longlong) INT_MAX);
+      adjusted= TRUE;
+    }
+    break;
+  case GET_LONG:
+#if SIZEOF_LONG < SIZEOF_LONG_LONG
+    if (num > (longlong) LONG_MAX)
+    {
+      num= ((longlong) LONG_MAX);
+      adjusted= TRUE;
+    }
+#endif
+    break;
+  default:
+    DBUG_ASSERT((optp->var_type & GET_TYPE_MASK) == GET_LL);
+    break;
+  }
+
   num= ((num - optp->sub_size) / block_size);
   num= (longlong) (num * block_size);
-  return max(num, optp->min_value);
+
+  if (num < optp->min_value)
+  {
+    num= optp->min_value;
+    adjusted= TRUE;
+  }
+
+  if (fix)
+    *fix= adjusted;
+  else if (adjusted)
+    my_getopt_error_reporter(WARNING_LEVEL,
+                             "option '%s': signed value %s adjusted to %s",
+                             optp->name, llstr(old, buf1), llstr(num, buf2));
+  return num;
 }
 
 /*
@@ -745,29 +863,173 @@ static longlong getopt_ll(char *arg, const struct my_option *optp, int *err)
 
 static ulonglong getopt_ull(char *arg, const struct my_option *optp, int *err)
 {
-  ulonglong num;
-
-  num= eval_num_suffix(arg, err, (char*) optp->name);  
-  return getopt_ull_limit_value(num, optp);
+  ulonglong num= eval_num_suffix(arg, err, (char*) optp->name);
+  return getopt_ull_limit_value(num, optp, NULL);
 }
 
 
-ulonglong getopt_ull_limit_value(ulonglong num, const struct my_option *optp)
+ulonglong getopt_ull_limit_value(ulonglong num, const struct my_option *optp,
+                                 my_bool *fix)
 {
+  my_bool adjusted= FALSE;
+  ulonglong old= num;
+  char buf1[255], buf2[255];
+
   if ((ulonglong) num > (ulonglong) optp->max_value &&
       optp->max_value) /* if max value is not set -> no upper limit */
+  {
     num= (ulonglong) optp->max_value;
+    adjusted= TRUE;
+  }
+
+  switch ((optp->var_type & GET_TYPE_MASK)) {
+  case GET_UINT:
+    if (num > (ulonglong) UINT_MAX)
+    {
+      num= ((ulonglong) UINT_MAX);
+      adjusted= TRUE;
+    }
+    break;
+  case GET_ULONG:
+#if SIZEOF_LONG < SIZEOF_LONG_LONG
+    if (num > (ulonglong) ULONG_MAX)
+    {
+      num= ((ulonglong) ULONG_MAX);
+      adjusted= TRUE;
+    }
+#endif
+    break;
+  default:
+    DBUG_ASSERT((optp->var_type & GET_TYPE_MASK) == GET_ULL);
+    break;
+  }
+
   if (optp->block_size > 1)
   {
     num/= (ulonglong) optp->block_size;
     num*= (ulonglong) optp->block_size;
   }
+
   if (num < (ulonglong) optp->min_value)
+  {
     num= (ulonglong) optp->min_value;
+    adjusted= TRUE;
+  }
+
+  if (fix)
+    *fix= adjusted;
+  else if (adjusted)
+    my_getopt_error_reporter(WARNING_LEVEL,
+                             "option '%s': unsigned value %s adjusted to %s",
+                             optp->name, ullstr(old, buf1), ullstr(num, buf2));
+
   return num;
 }
 
 
+/*
+  Get double value withing ranges
+
+  Evaluates and returns the value that user gave as an argument to a variable.
+
+  RETURN
+    decimal value of arg
+
+    In case of an error, prints an error message and sets *err to
+    EXIT_ARGUMENT_INVALID.  Otherwise err is not touched
+*/
+
+static double getopt_double(char *arg, const struct my_option *optp, int *err)
+{
+  double num;
+  int error;
+  char *end= arg + 1000;                     /* Big enough as *arg is \0 terminated */
+  num= my_strtod(arg, &end, &error);
+  if (end[0] != 0 || error)
+  {
+    fprintf(stderr,
+            "%s: ERROR: Invalid decimal value for option '%s'\n",
+            my_progname, optp->name);
+    *err= EXIT_ARGUMENT_INVALID;
+    return 0.0;
+  }
+  if (optp->max_value && num > (double) optp->max_value)
+    num= (double) optp->max_value;
+  return max(num, (double) optp->min_value);
+}
+
+/*
+  Init one value to it's default values
+
+  SYNOPSIS
+    init_one_value()
+    option		Option to initialize
+    value		Pointer to variable
+*/
+
+static void init_one_value(const struct my_option *option, uchar* *variable,
+			   longlong value)
+{
+  DBUG_ENTER("init_one_value");
+  switch ((option->var_type & GET_TYPE_MASK)) {
+  case GET_BOOL:
+    *((my_bool*) variable)= (my_bool) value;
+    break;
+  case GET_INT:
+    *((int*) variable)= (int) getopt_ll_limit_value((int) value, option, NULL);
+    break;
+  case GET_ENUM:
+    *((uint*) variable)= (uint) value;
+    break;
+  case GET_UINT:
+    *((uint*) variable)= (uint) getopt_ull_limit_value((uint) value, option, NULL);
+    break;
+  case GET_LONG:
+    *((long*) variable)= (long) getopt_ll_limit_value((long) value, option, NULL);
+    break;
+  case GET_ULONG:
+    *((ulong*) variable)= (ulong) getopt_ull_limit_value((ulong) value, option, NULL);
+    break;
+  case GET_LL:
+    *((longlong*) variable)= (longlong) getopt_ll_limit_value((longlong) value, option, NULL);
+    break;
+  case GET_ULL:
+  case GET_SET:
+    *((ulonglong*) variable)= (ulonglong) getopt_ull_limit_value((ulonglong) value, option, NULL);
+    break;
+  case GET_DOUBLE:
+    *((double*) variable)=  (double) value;
+    break;
+  case GET_STR:
+    /*
+      Do not clear variable value if it has no default value.
+      The default value may already be set.
+      NOTE: To avoid compiler warnings, we first cast longlong to intptr,
+      so that the value has the same size as a pointer.
+    */
+    if ((char*) (intptr) value)
+      *((char**) variable)= (char*) (intptr) value;
+    break;
+  case GET_STR_ALLOC:
+    /*
+      Do not clear variable value if it has no default value.
+      The default value may already be set.
+      NOTE: To avoid compiler warnings, we first cast longlong to intptr,
+      so that the value has the same size as a pointer.
+    */
+    if ((char*) (intptr) value)
+    {
+      my_free((*(char**) variable), MYF(MY_ALLOW_ZERO_PTR));
+      *((char**) variable)= my_strdup((char*) (intptr) value, MYF(MY_WME));
+    }
+    break;
+  default: /* dummy default to avoid compiler warnings */
+    break;
+  }
+  DBUG_VOID_RETURN;
+}
+
+
 /*
   Init one value to it's default values
 
@@ -777,34 +1039,25 @@ ulonglong getopt_ull_limit_value(ulonglong num, const struct my_option *optp)
     value		Pointer to variable
 */
 
-static void init_one_value(const struct my_option *option, gptr *variable,
-			   longlong value)
+static void fini_one_value(const struct my_option *option, uchar* *variable,
+			   longlong value __attribute__ ((unused)))
 {
+  DBUG_ENTER("fini_one_value");
   switch ((option->var_type & GET_TYPE_MASK)) {
-  case GET_BOOL:
-    *((my_bool*) variable)= (my_bool) value;
-    break;
-  case GET_INT:
-    *((int*) variable)= (int) value;
-    break;
-  case GET_UINT:
-    *((uint*) variable)= (uint) value;
-    break;
-  case GET_LONG:
-    *((long*) variable)= (long) value;
-    break;
-  case GET_ULONG:
-    *((ulong*) variable)= (ulong) value;
-    break;
-  case GET_LL:
-    *((longlong*) variable)= (longlong) value;
-    break;
-  case GET_ULL:
-    *((ulonglong*) variable)=  (ulonglong) value;
+  case GET_STR_ALLOC:
+    my_free((*(char**) variable), MYF(MY_ALLOW_ZERO_PTR));
+    *((char**) variable)= NULL;
     break;
   default: /* dummy default to avoid compiler warnings */
     break;
   }
+  DBUG_VOID_RETURN;
+}
+
+
+void my_cleanup_options(const struct my_option *options)
+{
+  init_variables(options, fini_one_value);
 }
 
 
@@ -821,11 +1074,14 @@ static void init_one_value(const struct my_option *option, gptr *variable,
     for a value and initialize.
 */
 
-static void init_variables(const struct my_option *options)
+static void init_variables(const struct my_option *options,
+                           init_func_p init_one_value)
 {
+  DBUG_ENTER("init_variables");
   for (; options->name; options++)
   {
-    gptr *variable;
+    uchar* *variable;
+    DBUG_PRINT("options", ("name: '%s'", options->name));
     /*
       We must set u_max_value first as for some variables
       options->u_max_value == options->value and in this case we want to
@@ -839,6 +1095,7 @@ static void init_variables(const struct my_option *options)
 	(variable= (*getopt_get_addr)("", 0, options)))
       init_one_value(options, variable, options->def_value);
   }
+  DBUG_VOID_RETURN;
 }
 
 
@@ -928,7 +1185,8 @@ void my_print_help(const struct my_option *options)
 
 void my_print_variables(const struct my_option *options)
 {
-  uint name_space= 34, length;
+  uint name_space= 34, length, nr;
+  ulonglong bit, llvalue;
   char buff[255];
   const struct my_option *optp;
 
@@ -937,15 +1195,30 @@ void my_print_variables(const struct my_option *options)
   printf("--------------------------------- -----------------------------\n");
   for (optp= options; optp->id; optp++)
   {
-    gptr *value= (optp->var_type & GET_ASK_ADDR ?
+    uchar* *value= (optp->var_type & GET_ASK_ADDR ?
 		  (*getopt_get_addr)("", 0, optp) : optp->value);
     if (value)
     {
-      printf("%s", optp->name);
-      length= (uint) strlen(optp->name);
+      printf("%s ", optp->name);
+      length= (uint) strlen(optp->name)+1;
       for (; length < name_space; length++)
 	putchar(' ');
       switch ((optp->var_type & GET_TYPE_MASK)) {
+      case GET_SET:
+        if (!(llvalue= *(ulonglong*) value))
+	  printf("%s\n", "(No default value)");
+	else
+        for (nr= 0, bit= 1; llvalue && nr < optp->typelib->count; nr++, bit<<=1)
+	{
+	  if (!(bit & llvalue))
+	    continue;
+	  llvalue&= ~bit;
+	  printf( llvalue ? "%s," : "%s\n", get_type(optp->typelib, nr));
+	}
+	break;
+      case GET_ENUM:
+        printf("%s\n", get_type(optp->typelib, *(uint*) value));
+	break;
       case GET_STR:
       case GET_STR_ALLOC:                    /* fall through */
 	printf("%s\n", *((char**) value) ? *((char**) value) :
@@ -961,7 +1234,7 @@ void my_print_variables(const struct my_option *options)
 	printf("%d\n", *((uint*) value));
 	break;
       case GET_LONG:
-	printf("%lu\n", *((long*) value));
+	printf("%ld\n", *((long*) value));
 	break;
       case GET_ULONG:
 	printf("%lu\n", *((ulong*) value));
@@ -973,6 +1246,9 @@ void my_print_variables(const struct my_option *options)
 	longlong2str(*((ulonglong*) value), buff, 10);
 	printf("%s\n", buff);
 	break;
+      case GET_DOUBLE:
+	printf("%g\n", *(double*) value);
+	break;
       default:
 	printf("(Disabled)\n");
 	break;
