@@ -1,9 +1,8 @@
-/* Copyright (C) 2004 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
+/* Copyright (C) 2004-2006 MySQL AB
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
- (at your option) any later version.
+ the Free Software Foundation; version 2 of the License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -55,24 +54,24 @@ uint calc_days_in_year(uint year)
           366 : 365);
 }
 
-/*
-  Check datetime value for validity according to flags.
+/**
+  @brief Check datetime value for validity according to flags.
 
-  SYNOPSIS
-    check_date()
-      ltime          Date to check.
-      not_zero_date  ltime is not the zero date
-      flags          flags to check
-      was_cut        set to 2 if value was truncated.
-		     NOTE: This is not touched if value was not truncated
-  NOTES
-    Here we assume that year and month is ok !
+  @param[in]  ltime          Date to check.
+  @param[in]  not_zero_date  ltime is not the zero date
+  @param[in]  flags          flags to check
+                             (see str_to_datetime() flags in my_time.h)
+  @param[out] was_cut        set to 2 if value was invalid according to flags.
+                             (Feb 29 in non-leap etc.)  This remains unchanged
+                             if value is not invalid.
+
+  @details Here we assume that year and month is ok!
     If month is 0 we allow any date. (This only happens if we allow zero
     date parts in str_to_datetime())
     Disallow dates with zero year and non-zero month and/or day.
 
-  RETURN
-    0  ok
+  @return
+    0  OK
     1  error
 */
 
@@ -86,8 +85,7 @@ my_bool check_date(const MYSQL_TIME *ltime, my_bool not_zero_date,
         (!(flags & TIME_INVALID_DATES) &&
          ltime->month && ltime->day > days_in_month[ltime->month-1] &&
          (ltime->month != 2 || calc_days_in_year(ltime->year) != 366 ||
-          ltime->day != 29)) ||
-        (ltime->year == 0 && (ltime->month != 0 || ltime->day != 0)))
+          ltime->day != 29)))
     {
       *was_cut= 2;
       return TRUE;
@@ -119,9 +117,9 @@ my_bool check_date(const MYSQL_TIME *ltime, my_bool not_zero_date,
                         TIME_NO_ZERO_IN_DATE	Don't allow partial dates
                         TIME_NO_ZERO_DATE	Don't allow 0000-00-00 date
                         TIME_INVALID_DATES	Allow 2000-02-31
-    was_cut             0	Value ok
+    was_cut             0	Value OK
 			1       If value was cut during conversion
-			2	Date part was within ranges but date was wrong
+			2	check_date(date,flags) considers date invalid
 
   DESCRIPTION
     At least the following formats are recogniced (based on number of digits)
@@ -207,7 +205,7 @@ str_to_datetime(const char *str, uint length, MYSQL_TIME *l_time,
   digits= (uint) (pos-str);
   start_loop= 0;                                /* Start of scan loop */
   date_len[format_position[0]]= 0;              /* Length of year field */
-  if (pos == end)
+  if (pos == end || *pos == '.')
   {
     /* Found date in internal format (only numbers like YYYYMMDD) */
     year_length= (digits == 4 || digits == 8 || digits >= 14) ? 4 : 2;
@@ -303,9 +301,6 @@ str_to_datetime(const char *str, uint length, MYSQL_TIME *l_time,
         field_length= 6;                        /* 6 digits */
       }
       continue;
-
-      /* No part seconds */
-      date[++i]= 0;
     }
     while (str != end &&
            (my_ispunct(&my_charset_latin1,*str) ||
@@ -411,9 +406,7 @@ str_to_datetime(const char *str, uint length, MYSQL_TIME *l_time,
   if (number_of_fields < 3 ||
       l_time->year > 9999 || l_time->month > 12 ||
       l_time->day > 31 || l_time->hour > 23 ||
-      l_time->minute > 59 || l_time->second > 59 ||
-      (l_time->year == 0 && l_time->month == 0 && l_time->day == 0 && 
-       (l_time->hour != 0 || l_time->minute != 0 || l_time->second != 0)))
+      l_time->minute > 59 || l_time->second > 59)
   {
     /* Only give warning for a zero date if there is some garbage after */
     if (!not_zero_date)                         /* If zero date */
@@ -457,7 +450,7 @@ err:
 
 
 /*
- Convert a time string to a TIME struct.
+ Convert a time string to a MYSQL_TIME struct.
 
   SYNOPSIS
    str_to_time()
@@ -573,12 +566,12 @@ my_bool str_to_time(const char *str, uint length, MYSQL_TIME *l_time,
     /* Fix the date to assume that seconds was given */
     if (!found_hours && !found_days)
     {
-      bmove_upp((char*) (date+4), (char*) (date+state),
+      bmove_upp((uchar*) (date+4), (uchar*) (date+state),
                 sizeof(long)*(state-1));
-      bzero((char*) date, sizeof(long)*(4-state));
+      bzero((uchar*) date, sizeof(long)*(4-state));
     }
     else
-      bzero((char*) (date+state), sizeof(long)*(4-state));
+      bzero((uchar*) (date+state), sizeof(long)*(4-state));
   }
 
 fractional:
@@ -643,11 +636,11 @@ fractional:
   l_time->second_part=  date[4];
   l_time->time_type= MYSQL_TIMESTAMP_TIME;
 
-  /* Check if the value is valid and fits into TIME range */
+  /* Check if the value is valid and fits into MYSQL_TIME range */
   if (check_time_range(l_time, warning))
     return 1;
   
-  /* Check if there is garbage at end of the TIME specification */
+  /* Check if there is garbage at end of the MYSQL_TIME specification */
   if (str != end)
   {
     do
@@ -664,11 +657,11 @@ fractional:
 
 
 /*
-  Check 'time' value to lie in the TIME range
+  Check 'time' value to lie in the MYSQL_TIME range
 
   SYNOPSIS:
     check_time_range()
-    time     pointer to TIME value
+    time     pointer to MYSQL_TIME value
     warning  set MYSQL_TIME_WARN_OUT_OF_RANGE flag if the value is out of range
 
   DESCRIPTION
@@ -681,24 +674,24 @@ fractional:
     1        time value is invalid
 */
 
-int check_time_range(struct st_mysql_time *time, int *warning) 
+int check_time_range(struct st_mysql_time *my_time, int *warning) 
 {
   longlong hour;
 
-  if (time->minute >= 60 || time->second >= 60)
+  if (my_time->minute >= 60 || my_time->second >= 60)
     return 1;
 
-  hour= time->hour + (24*time->day);
+  hour= my_time->hour + (24*my_time->day);
   if (hour <= TIME_MAX_HOUR &&
-      (hour != TIME_MAX_HOUR || time->minute != TIME_MAX_MINUTE ||
-       time->second != TIME_MAX_SECOND || !time->second_part))
+      (hour != TIME_MAX_HOUR || my_time->minute != TIME_MAX_MINUTE ||
+       my_time->second != TIME_MAX_SECOND || !my_time->second_part))
     return 0;
 
-  time->day= 0;
-  time->hour= TIME_MAX_HOUR;
-  time->minute= TIME_MAX_MINUTE;
-  time->second= TIME_MAX_SECOND;
-  time->second_part= 0;
+  my_time->day= 0;
+  my_time->hour= TIME_MAX_HOUR;
+  my_time->minute= TIME_MAX_MINUTE;
+  my_time->second= TIME_MAX_SECOND;
+  my_time->second_part= 0;
   *warning|= MYSQL_TIME_WARN_OUT_OF_RANGE;
   return 0;
 }
@@ -708,9 +701,9 @@ int check_time_range(struct st_mysql_time *time, int *warning)
   Prepare offset of system time zone from UTC for my_system_gmt_sec() func.
 
   SYNOPSIS
-    init_time()
+    my_init_time()
 */
-void init_time(void)
+void my_init_time(void)
 {
   time_t seconds;
   struct tm *l_time,tm_tmp;
@@ -731,30 +724,58 @@ void init_time(void)
 }
 
 
-	/* Calculate nr of day since year 0 in new date-system (from 1615) */
+/*
+  Handle 2 digit year conversions
+
+  SYNOPSIS
+  year_2000_handling()
+  year     2 digit year
+
+  RETURN
+    Year between 1970-2069
+*/
+
+uint year_2000_handling(uint year)
+{
+  if ((year=year+1900) < 1900+YY_PART_YEAR)
+    year+=100;
+  return year;
+}
+
+
+/*
+  Calculate nr of day since year 0 in new date-system (from 1615)
+
+  SYNOPSIS
+    calc_daynr()
+    year		 Year (exact 4 digit year, no year conversions)
+    month		 Month
+    day			 Day
+
+  NOTES: 0000-00-00 is a valid date, and will return 0
+
+  RETURN
+    Days since 0000-00-00
+*/
 
 long calc_daynr(uint year,uint month,uint day)
 {
   long delsum;
   int temp;
+  int y= year;                                  /* may be < 0 temporarily */
   DBUG_ENTER("calc_daynr");
 
-  if (year == 0 && month == 0 && day == 0)
+  if (y == 0 && month == 0 && day == 0)
     DBUG_RETURN(0);				/* Skip errors */
-  if (year < 200)
-  {
-    if ((year=year+1900) < 1900+YY_PART_YEAR)
-      year+=100;
-  }
-  delsum= (long) (365L * year+ 31*(month-1) +day);
+  delsum= (long) (365L * y+ 31*(month-1) +day);
   if (month <= 2)
-      year--;
+      y--;
   else
     delsum-= (long) (month*4+23)/10;
-  temp=(int) ((year/100+1)*3)/4;
+  temp=(int) ((y/100+1)*3)/4;
   DBUG_PRINT("exit",("year: %d  month: %d  day: %d -> daynr: %ld",
-		     year+(month <= 2),month,day,delsum+year/4-temp));
-  DBUG_RETURN(delsum+(int) year/4-temp);
+		     y+(month <= 2),month,day,delsum+y/4-temp));
+  DBUG_RETURN(delsum+(int) y/4-temp);
 } /* calc_daynr */
 
 
@@ -772,7 +793,7 @@ long calc_daynr(uint year,uint month,uint day)
   NOTES
     The idea is to cache the time zone offset from UTC (including daylight 
     saving time) for the next call to make things faster. But currently we 
-    just calculate this offset during startup (by calling init_time() 
+    just calculate this offset during startup (by calling my_init_time() 
     function) and use it all the time.
     Time value provided should be legal time value (e.g. '2003-01-01 25:00:00'
     is not allowed).
@@ -982,7 +1003,7 @@ void set_zero_time(MYSQL_TIME *tm, enum enum_mysql_timestamp_type time_type)
 /*
   Functions to convert time/date/datetime value to a string,
   using default format.
-  This functions don't check that given TIME structure members are
+  This functions don't check that given MYSQL_TIME structure members are
   in valid range. If they are not, return value won't reflect any
   valid date either. Additionally, make_time doesn't take into
   account time->day member: it's assumed that days have been converted
@@ -1064,11 +1085,11 @@ int my_TIME_to_str(const MYSQL_TIME *l_time, char *to)
       flags      - flags to use in validating date, as in str_to_datetime()
       was_cut    0      Value ok
                  1      If value was cut during conversion
-                 2      Date part was within ranges but date was wrong
+                 2      check_date(date,flags) considers date invalid
 
   DESCRIPTION
     Convert a datetime value of formats YYMMDD, YYYYMMDD, YYMMDDHHMSS,
-    YYYYMMDDHHMMSS to broken-down TIME representation. Return value in
+    YYYYMMDDHHMMSS to broken-down MYSQL_TIME representation. Return value in
     YYYYMMDDHHMMSS format as side-effect.
 
     This function also checks if datetime value fits in DATETIME range.
@@ -1085,9 +1106,14 @@ longlong number_to_datetime(longlong nr, MYSQL_TIME *time_res,
   long part1,part2;
 
   *was_cut= 0;
+  bzero((char*) time_res, sizeof(*time_res));
+  time_res->time_type=MYSQL_TIMESTAMP_DATE;
 
   if (nr == LL(0) || nr >= LL(10000101000000))
+  {
+    time_res->time_type=MYSQL_TIMESTAMP_DATETIME;
     goto ok;
+  }
   if (nr < 101)
     goto err;
   if (nr <= (YY_PART_YEAR-1)*10000L+1231L)
@@ -1111,6 +1137,9 @@ longlong number_to_datetime(longlong nr, MYSQL_TIME *time_res,
   }
   if (nr < 101000000L)
     goto err;
+
+  time_res->time_type=MYSQL_TIMESTAMP_DATETIME;
+
   if (nr <= (YY_PART_YEAR-1)*LL(10000000000)+LL(1231235959))
   {
     nr= nr+LL(20000000000000);                   /* YYMMDDHHMMSS, 2000-2069 */
@@ -1149,41 +1178,42 @@ longlong number_to_datetime(longlong nr, MYSQL_TIME *time_res,
 
 /* Convert time value to integer in YYYYMMDDHHMMSS format */
 
-ulonglong TIME_to_ulonglong_datetime(const MYSQL_TIME *time)
+ulonglong TIME_to_ulonglong_datetime(const MYSQL_TIME *my_time)
 {
-  return ((ulonglong) (time->year * 10000UL +
-                       time->month * 100UL +
-                       time->day) * ULL(1000000) +
-          (ulonglong) (time->hour * 10000UL +
-                       time->minute * 100UL +
-                       time->second));
+  return ((ulonglong) (my_time->year * 10000UL +
+                       my_time->month * 100UL +
+                       my_time->day) * ULL(1000000) +
+          (ulonglong) (my_time->hour * 10000UL +
+                       my_time->minute * 100UL +
+                       my_time->second));
 }
 
 
-/* Convert TIME value to integer in YYYYMMDD format */
+/* Convert MYSQL_TIME value to integer in YYYYMMDD format */
 
-ulonglong TIME_to_ulonglong_date(const MYSQL_TIME *time)
+ulonglong TIME_to_ulonglong_date(const MYSQL_TIME *my_time)
 {
-  return (ulonglong) (time->year * 10000UL + time->month * 100UL + time->day);
+  return (ulonglong) (my_time->year * 10000UL + my_time->month * 100UL +
+                      my_time->day);
 }
 
 
 /*
-  Convert TIME value to integer in HHMMSS format.
+  Convert MYSQL_TIME value to integer in HHMMSS format.
   This function doesn't take into account time->day member:
   it's assumed that days have been converted to hours already.
 */
 
-ulonglong TIME_to_ulonglong_time(const MYSQL_TIME *time)
+ulonglong TIME_to_ulonglong_time(const MYSQL_TIME *my_time)
 {
-  return (ulonglong) (time->hour * 10000UL +
-                      time->minute * 100UL +
-                      time->second);
+  return (ulonglong) (my_time->hour * 10000UL +
+                      my_time->minute * 100UL +
+                      my_time->second);
 }
 
 
 /*
-  Convert struct TIME (date and time split into year/month/day/hour/...
+  Convert struct MYSQL_TIME (date and time split into year/month/day/hour/...
   to a number in format YYYYMMDDHHMMSS (DATETIME),
   YYYYMMDD (DATE)  or HHMMSS (TIME).
 
@@ -1197,20 +1227,20 @@ ulonglong TIME_to_ulonglong_time(const MYSQL_TIME *time)
     SELECT ?+1;
 
   NOTE
-    This function doesn't check that given TIME structure members are
+    This function doesn't check that given MYSQL_TIME structure members are
     in valid range. If they are not, return value won't reflect any
     valid date either.
 */
 
-ulonglong TIME_to_ulonglong(const MYSQL_TIME *time)
+ulonglong TIME_to_ulonglong(const MYSQL_TIME *my_time)
 {
-  switch (time->time_type) {
+  switch (my_time->time_type) {
   case MYSQL_TIMESTAMP_DATETIME:
-    return TIME_to_ulonglong_datetime(time);
+    return TIME_to_ulonglong_datetime(my_time);
   case MYSQL_TIMESTAMP_DATE:
-    return TIME_to_ulonglong_date(time);
+    return TIME_to_ulonglong_date(my_time);
   case MYSQL_TIMESTAMP_TIME:
-    return TIME_to_ulonglong_time(time);
+    return TIME_to_ulonglong_time(my_time);
   case MYSQL_TIMESTAMP_NONE:
   case MYSQL_TIMESTAMP_ERROR:
     return ULL(0);
