@@ -1,4 +1,4 @@
-/* Copyright (c) 2005, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (C) 2005-2006 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,9 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
-/*
+/**
+  @file
+
   It is interface module to fixed precision decimals library.
 
   Most functions use 'uint mask' as parameter, if during operation error
@@ -34,13 +36,14 @@ C_MODE_END
 #define DECIMAL_LONG_DIGITS 10
 #define DECIMAL_LONG3_DIGITS 8
 
-/* maximum length of buffer in our big digits (uint32) */
+/** maximum length of buffer in our big digits (uint32). */
 #define DECIMAL_BUFF_LENGTH 9
 
 /* the number of digits that my_decimal can possibly contain */
 #define DECIMAL_MAX_POSSIBLE_PRECISION (DECIMAL_BUFF_LENGTH * 9)
 
-/*
+
+/**
   maximum guaranteed precision of number in decimal digits (number of our
   digits * number of decimal digits in one our big digit - number of decimal
   digits in one our big digit decreased by 1 (because we always put decimal
@@ -50,13 +53,14 @@ C_MODE_END
 #define DECIMAL_MAX_SCALE 30
 #define DECIMAL_NOT_SPECIFIED 31
 
-/*
+/**
   maximum length of string representation (number of maximum decimal
   digits + 1 position for sign + 1 position for decimal point)
 */
 #define DECIMAL_MAX_STR_LENGTH (DECIMAL_MAX_POSSIBLE_PRECISION + 2)
-/*
-  maximum size of packet length
+
+/**
+  maximum size of packet length.
 */
 #define DECIMAL_MAX_FIELD_SIZE DECIMAL_MAX_PRECISION
 
@@ -77,40 +81,22 @@ inline int my_decimal_int_part(uint precision, uint decimals)
 }
 
 
-/*
-  my_decimal class limits 'decimal_t' type to what we need in MySQL
+/**
+  my_decimal class limits 'decimal_t' type to what we need in MySQL.
+
   It contains internally all necessary space needed by the instance so
   no extra memory is needed. One should call fix_buffer_pointer() function
-  when he moves my_decimal objects in memory
+  when he moves my_decimal objects in memory.
 */
 
 class my_decimal :public decimal_t
 {
-  /*
-    Several of the routines in strings/decimal.c have had buffer
-    overrun/underrun problems. These are *not* caught by valgrind.
-    To catch them, we allocate dummy fields around the buffer,
-    and test that their values do not change.
-   */
-#if !defined(DBUG_OFF)
-  int foo1;
-#endif
-
   decimal_digit_t buffer[DECIMAL_BUFF_LENGTH];
 
-#if !defined(DBUG_OFF)
-  int foo2;
-  static const int test_value= 123;
-#endif
-
 public:
 
   void init()
   {
-#if !defined(DBUG_OFF)
-    foo1= test_value;
-    foo2= test_value;
-#endif
     len= DECIMAL_BUFF_LENGTH;
     buf= buffer;
 #if !defined (HAVE_purify) && !defined(DBUG_OFF)
@@ -123,28 +109,25 @@ public:
   {
     init();
   }
-  ~my_decimal()
-  {
-    sanity_check();
-  }
-
-  void sanity_check()
-  {
-    DBUG_ASSERT(foo1 == test_value);
-    DBUG_ASSERT(foo2 == test_value);
-  }
-
   void fix_buffer_pointer() { buf= buffer; }
 
   bool sign() const { return decimal_t::sign; }
   void sign(bool s) { decimal_t::sign= s; }
   uint precision() const { return intg + frac; }
+
+  /** Swap two my_decimal values */
+  void swap(my_decimal &rhs)
+  {
+    swap_variables(my_decimal, *this, rhs);
+    /* Swap the buffer pointers back */
+    swap_variables(decimal_digit_t *, buf, rhs.buf);
+  }
 };
 
 
 #ifndef DBUG_OFF
 void print_decimal(const my_decimal *dec);
-void print_decimal_buff(const my_decimal *dec, const byte* ptr, int length);
+void print_decimal_buff(const my_decimal *dec, const uchar* ptr, int length);
 const char *dbug_decimal_as_string(char *buff, const my_decimal *val);
 #else
 #define dbug_decimal_as_string(A) NULL
@@ -243,16 +226,15 @@ void my_decimal2decimal(const my_decimal *from, my_decimal *to)
 }
 
 
-int my_decimal2binary(uint mask, const my_decimal *d, char *bin, int prec,
+int my_decimal2binary(uint mask, const my_decimal *d, uchar *bin, int prec,
 		      int scale);
 
 
 inline
-int binary2my_decimal(uint mask, const char *bin, my_decimal *d, int prec,
+int binary2my_decimal(uint mask, const uchar *bin, my_decimal *d, int prec,
 		      int scale)
 {
-  return check_result(mask, bin2decimal((char *)bin, (decimal_t*) d, prec,
-					scale));
+  return check_result(mask, bin2decimal(bin, (decimal_t*) d, prec, scale));
 }
 
 
@@ -423,7 +405,10 @@ int my_decimal_mod(uint mask, my_decimal *res, const my_decimal *a,
 }
 
 
-/* Returns -1 if a<b, 1 if a>b and 0 if a==b */
+/**
+  @return
+    -1 if a<b, 1 if a>b and 0 if a==b
+*/
 inline
 int my_decimal_cmp(const my_decimal *a, const my_decimal *b)
 {
@@ -434,20 +419,11 @@ int my_decimal_cmp(const my_decimal *a, const my_decimal *b)
 inline
 int my_decimal_intg(const my_decimal *a)
 {
-    return decimal_intg((decimal_t*) a);
+  return decimal_intg((decimal_t*) a);
 }
 
 
-inline
-void my_decimal_trim(ulong *precision, uint *scale)
-{
-  if (!(*precision) && !(*scale))
-  {
-    *precision= 10;
-    *scale= 0;
-    return;
-  }
-}
+void my_decimal_trim(ulong *precision, uint *scale);
 
 
 #endif /*my_decimal_h*/
