@@ -1,4 +1,4 @@
-# Copyright (C) 2000-2007 MySQL AB
+# Copyright 2000-2008 MySQL AB, 2008 Sun Microsystems, Inc.
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,8 +14,12 @@
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston
 # MA  02110-1301  USA.
 
-%define mysql_version   @VERSION@
-%define mysql_vendor    MySQL AB
+%define mysql_version		@VERSION@
+
+# NOTE: "vendor" is used in upgrade/downgrade check, so you can't
+# change these, has to be exactly as is.
+%define mysql_old_vendor	MySQL AB
+%define mysql_vendor		Sun Microsystems, Inc.
 
 # use "rpmbuild --with static" or "rpm --define '_with_static 1'" (for RPM 3.x)
 # to enable static linking (off by default)
@@ -27,16 +31,21 @@
 %{?_with_yassl:%define YASSL_BUILD 1}
 %{!?_with_yassl:%define YASSL_BUILD 0}
 
+# use "rpmbuild --with cluster" or "rpm --define '_with_cluster 1'" (for RPM 3.x)
+# to build with cluster support (off by default)
+%{?_with_cluster:%define CLUSTER_BUILD 1}
+%{!?_with_cluster:%define CLUSTER_BUILD 0}
+
 %if %{STATIC_BUILD}
 %define release 0
 %else
 %define release 0.glibc23
 %endif
 %define license GPL
-%define mysqld_user     mysql
-%define mysqld_group    mysql
-%define server_suffix   -community
-%define mysqldatadir    /var/lib/mysql
+%define mysqld_user	mysql
+%define mysqld_group	mysql
+%define server_suffix -standard
+%define mysqldatadir /var/lib/mysql
 
 # We don't package all files installed into the build root by intention -
 # See BUG#998 for details.
@@ -66,10 +75,10 @@ Summary:	MySQL: a very fast and reliable SQL database server
 Group:		Applications/Databases
 Version:	@MYSQL_NO_DASH_VERSION@
 Release:	%{release}
-License:	%{license}
+License:	Copyright 2000-2008 MySQL AB, 2008 Sun Microsystems, Inc.  All rights reserved.  Use is subject to license terms.  Under %{license} license as shown in the Description field.
 Source:		http://www.mysql.com/Downloads/MySQL-@MYSQL_BASE_VERSION@/mysql-%{mysql_version}.tar.gz
 URL:		http://www.mysql.com/
-Packager:	MySQL Production Engineering Team <build@mysql.com>
+Packager:	Sun Microsystems, Inc. Product Engineering Team <build@mysql.com>
 Vendor:		%{mysql_vendor}
 Provides:	msqlormysql MySQL-server mysql
 BuildRequires: ncurses-devel
@@ -85,9 +94,11 @@ The MySQL(TM) software delivers a very fast, multi-threaded, multi-user,
 and robust SQL (Structured Query Language) database server. MySQL Server
 is intended for mission-critical, heavy-load production systems as well
 as for embedding into mass-deployed software. MySQL is a trademark of
-MySQL AB.
+Sun Microsystems, Inc.
+
+Copyright 2000-2008 MySQL AB, 2008 Sun Microsystems, Inc.  All rights reserved.
+Use is subject to license terms.
 
-Copyright (C) 2000-2007 MySQL AB
 This software comes with ABSOLUTELY NO WARRANTY. This is free software,
 and you are welcome to modify and redistribute it under the GPL license.
 
@@ -100,16 +111,18 @@ Summary:	MySQL: a very fast and reliable SQL database server
 Group:		Applications/Databases
 Requires: coreutils grep procps /usr/sbin/useradd /usr/sbin/groupadd /sbin/chkconfig
 Provides:	msqlormysql mysql-server mysql MySQL
-Obsoletes:	MySQL mysql mysql-server mysql-Max
+Obsoletes:	MySQL mysql mysql-server
 
 %description server
 The MySQL(TM) software delivers a very fast, multi-threaded, multi-user,
 and robust SQL (Structured Query Language) database server. MySQL Server
 is intended for mission-critical, heavy-load production systems as well
 as for embedding into mass-deployed software. MySQL is a trademark of
-MySQL AB.
+Sun Microsystems, Inc.
+
+Copyright 2000-2008 MySQL AB, 2008 Sun Microsystems, Inc.  All rights reserved.
+Use is subject to license terms.
 
-Copyright (C) 2000-2007 MySQL AB
 This software comes with ABSOLUTELY NO WARRANTY. This is free software,
 and you are welcome to modify and redistribute it under the GPL license.
 
@@ -121,7 +134,7 @@ This package includes the MySQL server binary (incl. InnoDB) as well
 as related utilities to run and administrate a MySQL server.
 
 If you want to access and work with the database, you have to install
-the package "MySQL-client" as well!
+package "MySQL-client" as well!
 
 %package client
 Summary: MySQL - Client
@@ -134,6 +147,7 @@ This package contains the standard MySQL clients and administration tools.
 
 %{see_base}
 
+%if %{CLUSTER_BUILD}
 %package ndb-storage
 Summary:	MySQL - ndbcluster storage engine
 Group:		Applications/Databases
@@ -174,17 +188,18 @@ This package contains some extra ndbcluster storage engine tools for the advance
 They should be used with caution.
 
 %{see_base}
+%endif
 
-%package bench
+%package test
 Requires: %{name}-client perl-DBI perl
-Summary: MySQL - Benchmarks and test system
+Summary: MySQL - Test suite
 Group: Applications/Databases
-Provides: mysql-bench
-Obsoletes: mysql-bench
+Provides: mysql-test
+Obsoletes: mysql-bench mysql-test
 AutoReqProv: no
 
-%description bench
-This package contains MySQL benchmark scripts and data.
+%description test
+This package contains the MySQL regression test suite.
 
 %{see_base}
 
@@ -203,34 +218,36 @@ necessary to develop MySQL client applications.
 %package shared
 Summary: MySQL - Shared libraries
 Group: Applications/Databases
-Provides: mysql-shared
-Obsoletes: mysql-shared
 
 %description shared
 This package contains the shared libraries (*.so*) which certain
 languages and applications need to dynamically load and use MySQL.
 
-#%package embedded
-#Requires: %{name}-devel
-#Summary: MySQL - embedded library
-#Group: Applications/Databases
-#Obsoletes: mysql-embedded
-#
-#%description embedded
-#This package contains the MySQL server as an embedded library.
-#
-#The embedded MySQL server library makes it possible to run a
-#full-featured MySQL server inside the client application.
-#The main benefits are increased speed and more simple management
-#for embedded applications.
-#
-#The API is identical for the embedded MySQL version and the
-#client/server version.
-#
-#%{see_base}
+%package embedded
+Requires: %{name}-devel
+Summary: MySQL - embedded library
+Group: Applications/Databases
+Obsoletes: mysql-embedded
+
+%description embedded
+This package contains the MySQL server as an embedded library.
+
+The embedded MySQL server library makes it possible to run a
+full-featured MySQL server inside the client application.
+The main benefits are increased speed and more simple management
+for embedded applications.
+
+The API is identical for the embedded MySQL version and the
+client/server version.
+
+%{see_base}
 
 %prep
-%setup -n mysql-%{mysql_version}
+# We unpack the source two times, for 'debug' and 'release' build.
+%setup -T -a 0 -c -n mysql-%{mysql_version}
+mv mysql-%{mysql_version} mysql-debug-%{mysql_version}
+%setup -D -T -a 0 -n mysql-%{mysql_version}
+mv mysql-%{mysql_version} mysql-release-%{mysql_version}
 
 %build
 
@@ -240,21 +257,28 @@ BuildMySQL() {
 sh -c  "PATH=\"${MYSQL_BUILD_PATH:-$PATH}\" \
 	CC=\"${CC:-$MYSQL_BUILD_CC}\" \
 	CXX=\"${CXX:-$MYSQL_BUILD_CXX}\" \
-	CFLAGS=\"${MYSQL_BUILD_CFLAGS:-$RPM_OPT_FLAGS}\" \
-	CXXFLAGS=\"${MYSQL_BUILD_CXXFLAGS:-$RPM_OPT_FLAGS \
-	          -felide-constructors -fno-exceptions -fno-rtti \
-		  }\" \
+	CFLAGS=\"$CFLAGS\" \
+	CXXFLAGS=\"$CXXFLAGS\" \
 	LDFLAGS=\"$MYSQL_BUILD_LDFLAGS\" \
 	./configure \
  	    $* \
+	    --with-mysqld-ldflags='-static' \
+	    --with-client-ldflags='-static' \
+	    --with-zlib-dir=bundled \
 	    --enable-assembler \
 	    --enable-local-infile \
+	    --with-fast-mutexes \
             --with-mysqld-user=%{mysqld_user} \
             --with-unix-socket-path=/var/lib/mysql/mysql.sock \
 	    --with-pic \
             --prefix=/ \
+%if %{CLUSTER_BUILD}
+	    --with-extra-charsets=all \
+%else
+	    --with-extra-charsets=complex \
+%endif
 %if %{YASSL_BUILD}
-	    --with-yassl \
+	    --with-ssl \
 %endif
             --exec-prefix=%{_exec_prefix} \
             --libexecdir=%{_sbindir} \
@@ -266,13 +290,22 @@ sh -c  "PATH=\"${MYSQL_BUILD_PATH:-$PATH}\" \
             --includedir=%{_includedir} \
             --mandir=%{_mandir} \
 	    --enable-thread-safe-client \
-	    --with-readline ; \
-	    # Add this for more debugging support
-	    # --with-debug
-	    "
-
- # benchdir does not fit in above model. Maybe a separate bench distribution
- make benchdir_root=$RPM_BUILD_ROOT/usr/share/
+	    --with-readline \
+		--with-innodb \
+%if %{CLUSTER_BUILD}
+		--with-ndbcluster \
+%else
+		--without-ndbcluster \
+%endif
+		--with-archive-storage-engine \
+		--with-csv-storage-engine \
+		--with-blackhole-storage-engine \
+		--with-federated-storage-engine \
+		--with-partition \
+		--with-big-tables \
+		--enable-shared \
+		"
+ make
 }
 
 # Use our own copy of glibc
@@ -287,7 +320,6 @@ fi
 # Use the build root for temporary storage of the shared libraries.
 
 RBR=$RPM_BUILD_ROOT
-MBD=$RPM_BUILD_DIR/mysql-%{mysql_version}
 
 # Clean up the BuildRoot first
 [ "$RBR" != "/" ] && [ -d $RBR ] && rm -rf $RBR;
@@ -299,6 +331,8 @@ mkdir -p $RBR%{_libdir}/mysql
 PATH=${MYSQL_BUILD_PATH:-/bin:/usr/bin}
 export PATH
 
+# Build the Debug binary.
+
 # Use gcc for C and C++ code (to avoid a dependency on libstdc++ and
 # including exceptions into the code
 if [ -z "$CXX" -a -z "$CC" ]
@@ -307,46 +341,62 @@ then
 	export CXX="gcc"
 fi
 
-#
-# Only link statically on our i386 build host (which has a specially
-# patched static glibc installed) - ia64 and x86_64 run glibc-2.3 (unpatched)
-# so don't link statically there
-#
-for servertype in '--with-debug=full' ' '
-do
-  BuildMySQL "\
-%if %{STATIC_BUILD}
-		--enable-shared \
-		--with-mysqld-ldflags='-all-static' \
-		--with-client-ldflags='-all-static' \
-		$USE_OTHER_LIBC_DIR \
-%else
-		--enable-shared \
-		--with-zlib-dir=bundled \
-%endif
-		--with-extra-charsets=complex \
-		--with-comment=\"MySQL Community Edition (GPL)\" \
-		--with-server-suffix='%{server_suffix}' \
-		--with-archive-storage-engine \
-		--with-innodb \
-		--with-ndbcluster \
-		--with-csv-storage-engine \
-		--with-example-storage-engine \
-		--with-blackhole-storage-engine \
-		--with-federated-storage-engine \
-		--with-big-tables $servertype"
-  if test "$servertype" != ' '
-  then
-    # if this is not the regular build, we save the server binary
-    ./libtool --mode=execute cp sql/mysqld sql/mysqld-debug
-    ./libtool --mode=execute nm --numeric-sort sql/mysqld-debug > sql/mysqld-debug.sym
-    echo "# debug"
-    make test-bt-debug
-    make clean
-  fi
-done
+# Prepare compiler flags
+CFLAGS=${MYSQL_BUILD_CFLAGS:-$RPM_OPT_FLAGS}
+CXXFLAGS=${MYSQL_BUILD_CXXFLAGS:-$RPM_OPT_FLAGS -felide-constructors -fno-exceptions -fno-rtti }
 
-./libtool --mode=execute nm --numeric-sort sql/mysqld > sql/mysqld.sym
+##############################################################################
+#
+#  Build the debug version
+#
+##############################################################################
+
+(
+# We are in a subshell, so we can modify variables just for one run.
+CFLAGS=`echo   " $CFLAGS "   | \
+    sed -e 's/ -O[0-9]* / /' -e 's/ -unroll2 / /' -e 's/ -ip / /' \
+        -e 's/^ //' -e 's/ $//'`
+CXXFLAGS=`echo " $CXXFLAGS " | \
+    sed -e 's/ -O[0-9]* / /' -e 's/ -unroll2 / /' -e 's/ -ip / /' \
+        -e 's/^ //' -e 's/ $//'`
+
+# Add -g and --with-debug.
+cd mysql-debug-%{mysql_version} &&
+CFLAGS="$CFLAGS" \
+CXXFLAGS="$CXXFLAGS" \
+BuildMySQL "\
+		--with-debug \
+		--with-comment=\"MySQL Community Server - Debug (%{license})\"")
+
+# We might want to save the config log file
+if test -n "$MYSQL_DEBUGCONFLOG_DEST"
+then
+  cp -fp mysql-debug-%{mysql_version}/config.log "$MYSQL_DEBUGCONFLOG_DEST"
+fi
+
+(cd mysql-debug-%{mysql_version} ; make test-bt-debug)
+
+##############################################################################
+#
+#  Build the release binary
+#
+##############################################################################
+
+(cd mysql-release-%{mysql_version} &&
+CFLAGS="$CFLAGS" \
+CXXFLAGS="$CXXFLAGS" \
+BuildMySQL "\
+		--with-embedded-server \
+		--with-comment=\"MySQL Community Server (%{license})\"")
+# We might want to save the config log file
+if test -n "$MYSQL_CONFLOG_DEST"
+then
+  cp -fp  mysql-release-%{mysql_version}/config.log "$MYSQL_CONFLOG_DEST"
+fi
+
+(cd mysql-release-%{mysql_version} ; make test-bt)
+
+##############################################################################
 
 # Include libgcc.a in the devel subpackage (BUG 4921)
 if expr "$CC" : ".*gcc.*" > /dev/null ;
@@ -359,66 +409,50 @@ then
   fi
 fi
 
-# Save the libraries
-(cd libmysql/.libs; tar cf $RBR/shared-libs.tar *.so*)
-(cd libmysql_r/.libs; tar rf $RBR/shared-libs.tar *.so*)
-(cd ndb/src/.libs; tar rf $RBR/shared-libs.tar *.so*)
-
-# We might want to save the config log file
-if test -n "$MYSQL_CONFLOG_DEST"
-then
-  cp -fp config.log "$MYSQL_CONFLOG_DEST"
-fi
-
-echo "# standard"
-make test-bt
+##############################################################################
 
 %install
 RBR=$RPM_BUILD_ROOT
-MBD=$RPM_BUILD_DIR/mysql-%{mysql_version}
+MBD=$RPM_BUILD_DIR/mysql-%{mysql_version}/mysql-release-%{mysql_version}
 
 # Ensure that needed directories exists
 install -d $RBR%{_sysconfdir}/{logrotate.d,init.d}
 install -d $RBR%{mysqldatadir}/mysql
-install -d $RBR%{_datadir}/{sql-bench,mysql-test}
+install -d $RBR%{_datadir}/mysql-test
 install -d $RBR%{_includedir}
 install -d $RBR%{_libdir}
 install -d $RBR%{_mandir}
 install -d $RBR%{_sbindir}
 
-# Install all binaries stripped 
-make install-strip DESTDIR=$RBR benchdir_root=%{_datadir}
 
-# Install the ndb binaries
-(cd ndb; make install DESTDIR=$RBR)
+# Install all binaries 
+(cd $MBD && make install DESTDIR=$RBR benchdir_root=%{_datadir})
+# Old packages put shared libs in %{_libdir}/ (not %{_libdir}/mysql), so do
+# the same here.
+mv $RBR/%{_libdir}/mysql/*.so* $RBR/%{_libdir}/
 
-# Install the saved debug server
-install -s -m 755 $MBD/sql/mysqld-debug $RBR%{_sbindir}/mysqld-debug
+# install "mysqld-debug"
+$MBD/libtool --mode=execute install -m 755 \
+                 $RPM_BUILD_DIR/mysql-%{mysql_version}/mysql-debug-%{mysql_version}/sql/mysqld \
+                 $RBR%{_sbindir}/mysqld-debug
 
-# Install shared libraries (Disable for architectures that don't support it)
-(cd $RBR%{_libdir}; tar xf $RBR/shared-libs.tar; rm -f $RBR/shared-libs.tar)
-
-# install symbol files ( for stack trace resolution)
-# install -m 644 $MBD/sql/mysqld-max.sym $RBR%{_libdir}/mysql/mysqld-max.sym
-install -m 644 $MBD/sql/mysqld.sym $RBR%{_libdir}/mysql/mysqld.sym
-install -m 644 $MBD/sql/mysqld-debug.sym $RBR%{_libdir}/mysql/mysqld-debug.sym
+# install saved perror binary with NDB support (BUG#13740)
+install -m 755 $MBD/extra/perror $RBR%{_bindir}/perror
 
 # Install logrotate and autostart
 install -m 644 $MBD/support-files/mysql-log-rotate $RBR%{_sysconfdir}/logrotate.d/mysql
 install -m 755 $MBD/support-files/mysql.server $RBR%{_sysconfdir}/init.d/mysql
 
 # Install embedded server library in the build root
-# FIXME No libmysqld on 5.0 yet
-#install -m 644 libmysqld/libmysqld.a $RBR%{_libdir}/mysql/
+install -m 644 $MBD/libmysqld/libmysqld.a $RBR%{_libdir}/mysql/
+
+# in RPMs, it is unlikely that anybody should use "sql-bench"
+rm -fr $RBR%{_datadir}/sql-bench
 
 # Create a symlink "rcmysql", pointing to the init.script. SuSE users
 # will appreciate that, as all services usually offer this.
 ln -s %{_sysconfdir}/init.d/mysql $RPM_BUILD_ROOT%{_sbindir}/rcmysql
 
-# Create symbolic compatibility link safe_mysqld -> mysqld_safe
-# (safe_mysqld will be gone in MySQL 4.1)
-ln -sf ./mysqld_safe $RBR%{_bindir}/safe_mysqld
-
 # Touch the place where the my.cnf config file and mysqlmanager.passwd
 # (MySQL Instance Manager password file) might be located
 # Just to make sure it's in the file list and marked as a config file
@@ -433,6 +467,7 @@ installed=`rpm -q --whatprovides mysql-server 2> /dev/null`
 if [ $? -eq 0 -a -n "$installed" ]; then
   vendor=`rpm -q --queryformat='%{VENDOR}' "$installed" 2>&1`
   version=`rpm -q --queryformat='%{VERSION}' "$installed" 2>&1`
+  myoldvendor='%{mysql_old_vendor}'
   myvendor='%{mysql_vendor}'
   myversion='%{mysql_version}'
 
@@ -444,12 +479,12 @@ if [ $? -eq 0 -a -n "$installed" ]; then
   [ -z "$new_family" ] && new_family="<bad package specification: version $myversion>"
 
   error_text=
-  if [ "$vendor" != "$myvendor" ]; then
+  if [ "$vendor" != "$myoldvendor" -a "$vendor" != "$myvendor" ]; then
     error_text="$error_text
 The current MySQL server package is provided by a different
-vendor ($vendor) than $myvendor.  Some files may be installed
-to different locations, including log files and the service
-startup script in %{_sysconfdir}/init.d/.
+vendor ($vendor) than $myoldvendor or $myvendor.
+Some files may be installed to different locations, including log
+files and the service startup script in %{_sysconfdir}/init.d/.
 "
   fi
 
@@ -550,20 +585,22 @@ chmod -R og-rw $mysql_datadir/mysql
 # Restart in the same way that mysqld will be started normally.
 %{_sysconfdir}/init.d/mysql start
 
-# Allow safe_mysqld to start mysqld and print a message before we exit
+# Allow mysqld_safe to start mysqld and print a message before we exit
 sleep 2
 
-echo "Thank you for installing the MySQL Community Server! For Production
-systems, we recommend MySQL Enterprise, which contains enterprise-ready
-software, intelligent advisory services, and full production support with
-scheduled service packs and more.  Visit www.mysql.com/enterprise for more
-information." 
+#echo "Thank you for installing the MySQL Community Server! For Production
+#systems, we recommend MySQL Enterprise, which contains enterprise-ready
+#software, intelligent advisory services, and full production support with
+#scheduled service packs and more.  Visit www.mysql.com/enterprise for more
+#information." 
 
+%if %{CLUSTER_BUILD}
 %post ndb-storage
 mysql_clusterdir=/var/lib/mysql-cluster
 
 # Create cluster directory if needed
 if test ! -d $mysql_clusterdir; then mkdir -m 755 $mysql_clusterdir; fi
+%endif
 
 %preun server
 if test $1 = 0
@@ -596,18 +633,20 @@ fi
 %files server
 %defattr(-,root,root,0755)
 
-%doc COPYING README 
-%doc support-files/my-*.cnf
-%doc support-files/ndb-*.ini
+%doc mysql-release-%{mysql_version}/COPYING mysql-release-%{mysql_version}/README 
+%doc mysql-release-%{mysql_version}/support-files/my-*.cnf
+%if %{CLUSTER_BUILD}
+%doc mysql-release-%{mysql_version}/support-files/ndb-*.ini
+%endif
 
 %doc %attr(644, root, root) %{_infodir}/mysql.info*
 
+%doc %attr(644, root, man) %{_mandir}/man1/innochecksum.1*
 %doc %attr(644, root, man) %{_mandir}/man1/my_print_defaults.1*
 %doc %attr(644, root, man) %{_mandir}/man1/myisam_ftdump.1*
 %doc %attr(644, root, man) %{_mandir}/man1/myisamchk.1*
 %doc %attr(644, root, man) %{_mandir}/man1/myisamlog.1*
 %doc %attr(644, root, man) %{_mandir}/man1/myisampack.1*
-%doc %attr(644, root, man) %{_mandir}/man1/mysql_explain_log.1*
 %doc %attr(644, root, man) %{_mandir}/man8/mysqld.8*
 %doc %attr(644, root, man) %{_mandir}/man1/mysqld_multi.1*
 %doc %attr(644, root, man) %{_mandir}/man1/mysqld_safe.1*
@@ -621,20 +660,20 @@ fi
 %doc %attr(644, root, man) %{_mandir}/man1/mysqltest.1*
 %doc %attr(644, root, man) %{_mandir}/man1/mysql_tzinfo_to_sql.1*
 %doc %attr(644, root, man) %{_mandir}/man1/mysql_zap.1*
+%doc %attr(644, root, man) %{_mandir}/man1/mysqlbug.1*
 %doc %attr(644, root, man) %{_mandir}/man1/perror.1*
 %doc %attr(644, root, man) %{_mandir}/man1/replace.1*
-%doc %attr(644, root, man) %{_mandir}/man1/safe_mysqld.1*
 
 %ghost %config(noreplace,missingok) %{_sysconfdir}/my.cnf
 %ghost %config(noreplace,missingok) %{_sysconfdir}/mysqlmanager.passwd
 
+%attr(755, root, root) %{_bindir}/innochecksum
 %attr(755, root, root) %{_bindir}/my_print_defaults
 %attr(755, root, root) %{_bindir}/myisam_ftdump
 %attr(755, root, root) %{_bindir}/myisamchk
 %attr(755, root, root) %{_bindir}/myisamlog
 %attr(755, root, root) %{_bindir}/myisampack
 %attr(755, root, root) %{_bindir}/mysql_convert_table_format
-%attr(755, root, root) %{_bindir}/mysql_explain_log
 %attr(755, root, root) %{_bindir}/mysql_fix_extensions
 %attr(755, root, root) %{_bindir}/mysql_fix_privilege_tables
 %attr(755, root, root) %{_bindir}/mysql_install_db
@@ -653,13 +692,11 @@ fi
 %attr(755, root, root) %{_bindir}/replace
 %attr(755, root, root) %{_bindir}/resolve_stack_dump
 %attr(755, root, root) %{_bindir}/resolveip
-%attr(755, root, root) %{_bindir}/safe_mysqld
 
 %attr(755, root, root) %{_sbindir}/mysqld
 %attr(755, root, root) %{_sbindir}/mysqld-debug
 %attr(755, root, root) %{_sbindir}/mysqlmanager
 %attr(755, root, root) %{_sbindir}/rcmysql
-%attr(644, root, root) %{_libdir}/mysql/mysqld.sym
 
 %attr(644, root, root) %config(noreplace,missingok) %{_sysconfdir}/logrotate.d/mysql
 %attr(755, root, root) %{_sysconfdir}/init.d/mysql
@@ -671,7 +708,6 @@ fi
 %attr(755, root, root) %{_bindir}/msql2mysql
 %attr(755, root, root) %{_bindir}/mysql
 %attr(755, root, root) %{_bindir}/mysql_find_rows
-%attr(755, root, root) %{_bindir}/mysql_tableinfo
 %attr(755, root, root) %{_bindir}/mysql_waitpid
 %attr(755, root, root) %{_bindir}/mysqlaccess
 %attr(755, root, root) %{_bindir}/mysqladmin
@@ -680,9 +716,11 @@ fi
 %attr(755, root, root) %{_bindir}/mysqldump
 %attr(755, root, root) %{_bindir}/mysqlimport
 %attr(755, root, root) %{_bindir}/mysqlshow
+%attr(755, root, root) %{_bindir}/mysqlslap
 
 %doc %attr(644, root, man) %{_mandir}/man1/msql2mysql.1*
 %doc %attr(644, root, man) %{_mandir}/man1/mysql.1*
+%doc %attr(644, root, man) %{_mandir}/man1/mysql_find_rows.1*
 %doc %attr(644, root, man) %{_mandir}/man1/mysqlaccess.1*
 %doc %attr(644, root, man) %{_mandir}/man1/mysqladmin.1*
 %doc %attr(644, root, man) %{_mandir}/man1/mysqlbinlog.1*
@@ -690,6 +728,7 @@ fi
 %doc %attr(644, root, man) %{_mandir}/man1/mysqldump.1*
 %doc %attr(644, root, man) %{_mandir}/man1/mysqlimport.1*
 %doc %attr(644, root, man) %{_mandir}/man1/mysqlshow.1*
+%doc %attr(644, root, man) %{_mandir}/man1/mysqlslap.1*
 
 %post shared
 /sbin/ldconfig
@@ -697,6 +736,7 @@ fi
 %postun shared
 /sbin/ldconfig
 
+%if %{CLUSTER_BUILD}
 %files ndb-storage
 %defattr(-,root,root,0755)
 %attr(755, root, root) %{_sbindir}/ndbd
@@ -713,6 +753,9 @@ fi
 %attr(755, root, root) %{_bindir}/ndb_desc
 %attr(755, root, root) %{_bindir}/ndb_error_reporter
 %attr(755, root, root) %{_bindir}/ndb_mgm
+%attr(755, root, root) %{_bindir}/ndb_print_backup_file
+%attr(755, root, root) %{_bindir}/ndb_print_schema_file
+%attr(755, root, root) %{_bindir}/ndb_print_sys_file
 %attr(755, root, root) %{_bindir}/ndb_restore
 %attr(755, root, root) %{_bindir}/ndb_select_all
 %attr(755, root, root) %{_bindir}/ndb_select_count
@@ -720,10 +763,14 @@ fi
 %attr(755, root, root) %{_bindir}/ndb_size.pl
 %attr(755, root, root) %{_bindir}/ndb_test_platform
 %attr(755, root, root) %{_bindir}/ndb_waiter
-%attr(-, root, root) %{_datadir}/mysql/ndb_size.tmpl
 %doc %attr(644, root, man) %{_mandir}/man1/ndb_config.1*
 %doc %attr(644, root, man) %{_mandir}/man1/ndb_desc.1*
 %doc %attr(644, root, man) %{_mandir}/man1/ndb_error_reporter.1*
+%doc %attr(644, root, man) %{_mandir}/man1/ndb_mgm.1*
+%doc %attr(644, root, man) %{_mandir}/man1/ndb_restore.1*
+%doc %attr(644, root, man) %{_mandir}/man1/ndb_print_backup_file.1*
+%doc %attr(644, root, man) %{_mandir}/man1/ndb_print_schema_file.1*
+%doc %attr(644, root, man) %{_mandir}/man1/ndb_print_sys_file.1*
 %doc %attr(644, root, man) %{_mandir}/man1/ndb_select_all.1*
 %doc %attr(644, root, man) %{_mandir}/man1/ndb_select_count.1*
 %doc %attr(644, root, man) %{_mandir}/man1/ndb_show_tables.1*
@@ -735,19 +782,22 @@ fi
 %attr(755, root, root) %{_bindir}/ndb_delete_all
 %attr(755, root, root) %{_bindir}/ndb_drop_index
 %attr(755, root, root) %{_bindir}/ndb_drop_table
+%attr(755, root, root) %{_sbindir}/ndb_cpcd
 %doc %attr(644, root, man) %{_mandir}/man1/ndb_delete_all.1*
 %doc %attr(644, root, man) %{_mandir}/man1/ndb_drop_index.1*
 %doc %attr(644, root, man) %{_mandir}/man1/ndb_drop_table.1*
+%doc %attr(644, root, man) %{_mandir}/man1/ndb_cpcd.1*
+%endif
 
 %files devel
 %defattr(-, root, root, 0755)
-%doc EXCEPTIONS-CLIENT
+%doc mysql-release-%{mysql_version}/EXCEPTIONS-CLIENT
 %doc %attr(644, root, man) %{_mandir}/man1/mysql_config.1*
-%attr(755, root, root) %{_bindir}/comp_err
 %attr(755, root, root) %{_bindir}/mysql_config
 %dir %attr(755, root, root) %{_includedir}/mysql
 %dir %attr(755, root, root) %{_libdir}/mysql
 %{_includedir}/mysql/*
+%{_datadir}/aclocal/mysql.m4
 %{_libdir}/mysql/libdbug.a
 %{_libdir}/mysql/libheap.a
 %if %{have_libgcc}
@@ -761,40 +811,80 @@ fi
 %{_libdir}/mysql/libmysqlclient_r.la
 %{_libdir}/mysql/libmystrings.a
 %{_libdir}/mysql/libmysys.a
+%if %{CLUSTER_BUILD}
 %{_libdir}/mysql/libndbclient.a
 %{_libdir}/mysql/libndbclient.la
+%endif
 %{_libdir}/mysql/libvio.a
-%if %{STATIC_BUILD}
-%else
 %{_libdir}/mysql/libz.a
 %{_libdir}/mysql/libz.la
-%endif
 
 %files shared
 %defattr(-, root, root, 0755)
 # Shared libraries (omit for architectures that don't support them)
-%{_libdir}/*.so*
+%{_libdir}/libmysql*.so*
+%if %{CLUSTER_BUILD}
+%{_libdir}/libndb*.so*
+%endif
 
-%files bench
+%files test
 %defattr(-, root, root, 0755)
-%attr(-, root, root) %{_datadir}/sql-bench
 %attr(-, root, root) %{_datadir}/mysql-test
 %attr(755, root, root) %{_bindir}/mysql_client_test
-%attr(755, root, root) %{_bindir}/mysqltestmanager
-%attr(755, root, root) %{_bindir}/mysqltestmanager-pwgen
-%attr(755, root, root) %{_bindir}/mysqltestmanagerc
+%attr(755, root, root) %{_bindir}/mysql_client_test_embedded
+%attr(755, root, root) %{_bindir}/mysqltest_embedded
 %doc %attr(644, root, man) %{_mandir}/man1/mysql_client_test.1*
+%doc %attr(644, root, man) %{_mandir}/man1/mysql_client_test_embedded.1*
 %doc %attr(644, root, man) %{_mandir}/man1/mysql-stress-test.pl.1*
 %doc %attr(644, root, man) %{_mandir}/man1/mysql-test-run.pl.1*
+%doc %attr(644, root, man) %{_mandir}/man1/mysqltest_embedded.1*
 
-#%files embedded
-#%defattr(-, root, root, 0755)
-# %attr(644, root, root) %{_libdir}/mysql/libmysqld.a
+%files embedded
+%defattr(-, root, root, 0755) 
+%attr(644, root, root) %{_libdir}/mysql/libmysqld.a
 
 # The spec file changelog only includes changes made to the spec file
 # itself - note that they must be ordered by date (important when
 # merging BK trees)
 %changelog
+* Fri Nov 07 2008 Joerg Bruehe <joerg@mysql.com>
+
+- Correct yesterday's fix, so that it also works for the last flag,
+  and fix a wrong quoting: un-quoted quote marks must not be escaped.
+  
+* Thu Nov 06 2008 Kent Boortz <kent.boortz@sun.com>
+
+- Removed "mysql_upgrade_shell"
+- Removed some copy/paste between debug and normal build
+
+* Thu Nov 06 2008 Joerg Bruehe <joerg@mysql.com>
+
+- Modify CFLAGS and CXXFLAGS such that a debug build is not optimized.
+  This should cover both gcc and icc flags.  Fixes bug#40546.
+  
+* Fri Aug 29 2008 Kent Boortz <kent@mysql.com>
+
+- Removed the "Federated" storage engine option, and enabled in all
+
+* Tue Aug 26 2008 Joerg Bruehe <joerg@mysql.com>
+
+- Get rid of the "warning: Installed (but unpackaged) file(s) found:"
+  Some generated files aren't needed in RPMs:
+  - the "sql-bench/" subdirectory
+  Some files were missing:
+  - /usr/share/aclocal/mysql.m4  ("devel" subpackage)
+  - Manual "mysqlbug" ("server" subpackage)
+  - Program "innochecksum" and its manual ("server" subpackage)
+  - Manual "mysql_find_rows" ("client" subpackage)
+  - Script "mysql_upgrade_shell" ("client" subpackage)
+  - Program "ndb_cpcd" and its manual ("ndb-extra" subpackage)
+  - Manuals "ndb_mgm" + "ndb_restore" ("ndb-tools" subpackage)
+
+* Mon Mar 31 2008 Kent Boortz <kent@mysql.com>
+
+- Made the "Federated" storage engine an option
+- Made the "Cluster" storage engine and sub packages an option
+
 * Wed Mar 19 2008 Joerg Bruehe <joerg@mysql.com>
 
 - Add the man pages for "ndbd" and "ndb_mgmd".
@@ -804,9 +894,10 @@ fi
 - Require a manual upgrade if the alread-installed mysql-server is
   from another vendor, or is of a different major version.
 
-* Fri Nov 16 2007 Joerg Bruehe <joerg@mysql.com>
+* Wed May 02 2007 Joerg Bruehe <joerg@mysql.com>
 
-- When testing the debug server, use "make test-bt-debug".
+- "ndb_size.tmpl" is not needed any more, 
+  "man1/mysql_install_db.1" lacked the trailing '*'.
 
 * Sat Apr 07 2007 Kent Boortz <kent@mysql.com>
 
@@ -825,12 +916,10 @@ fi
 
 - Add several man pages for NDB which are now created.
 
-* Wed Jan 31 2007 Daniel Fischer <df@mysql.com>
-
-- add MTR_BUILD_THREAD=auto to test runs.
-
 * Fri Jan 05 2007 Kent Boortz <kent@mysql.com>
 
+- Put back "libmygcc.a", found no real reason it was removed.
+
 - Add CFLAGS to gcc call with --print-libgcc-file, to make sure the
   correct "libgcc.a" path is returned for the 32/64 bit architecture.
 
@@ -844,16 +933,27 @@ fi
   in the server RPM.
 - The "mysqlmanager" man page got moved from section 1 to 8.
 
+* Thu Nov 30 2006 Joerg Bruehe <joerg@mysql.com>
+
+- Call "make install" using "benchdir_root=%{_datadir}", 
+  because that is affecting the regression test suite as well.
+
 * Thu Nov 16 2006 Joerg Bruehe <joerg@mysql.com>
 
 - Explicitly note that the "MySQL-shared" RPMs (as built by MySQL AB) 
   replace "mysql-shared" (as distributed by SuSE) to allow easy upgrading
   (bug#22081).
 
-* Wed Nov 15 2006 Joerg Bruehe <joerg@mysql.com>
+* Mon Nov 13 2006 Joerg Bruehe <joerg@mysql.com>
 
-- Switch from "make test*" to explicit calls of the test suite,
-  so that "report features" can be used.
+- Add "--with-partition" to all server builds.
+
+- Use "--report-features" in one test run per server build.
+
+* Tue Aug 15 2006 Joerg Bruehe <joerg@mysql.com>
+
+- The "max" server is removed from packages, effective from 5.1.12-beta.
+  Delete all steps to build, package, or install it.
 
 * Mon Jul 10 2006 Joerg Bruehe <joerg@mysql.com>
 
@@ -889,9 +989,9 @@ fi
 
 * Wed May 10 2006 Kent Boortz <kent@mysql.com>
 
-- Use character set "all" for the "max", to make Cluster nodes
-  independent on the character set directory, and the problem that
-  two RPM sub packages both wants to install this directory.
+- Use character set "all" when compiling with Cluster, to make Cluster
+  nodes independent on the character set directory, and the problem
+  that two RPM sub packages both wants to install this directory.
 
 * Mon May 01 2006 Kent Boortz <kent@mysql.com>
 
@@ -902,25 +1002,52 @@ fi
 
 - Install and run "mysql_upgrade"
 
+* Wed Apr 12 2006 Jim Winstead <jimw@mysql.com>
+
+- Remove sql-bench, and MySQL-bench RPM (will be built as an independent
+  project from the mysql-bench repository)
+
+* Tue Apr 11 2006 Jim Winstead <jimw@mysql.com>
+
+- Remove old mysqltestmanager and related programs
 * Sat Apr 01 2006 Kent Boortz <kent@mysql.com>
 
 - Set $LDFLAGS from $MYSQL_BUILD_LDFLAGS
 
-* Fri Mar 03 2006 Kent Boortz <kent@mysql.com>
+* Wed Mar 07 2006 Kent Boortz <kent@mysql.com>
 
-- Don't output an embedded package as it is empty
-- Can't use bundled zlib when doing static build. Might be a
-  automake/libtool problem, having two .la files, "libmysqlclient.la"
-  and "libz.la", on the same command line to link "thread_test"
-  expands to too many "-lc", "-lpthread" and other libs giving hard
-  to nail down duplicate symbol defintion problems.
+- Changed product name from "Community Edition" to "Community Server"
 
-* Fri Jan 10 2006 Joerg Bruehe <joerg@mysql.com>
+* Mon Mar 06 2006 Kent Boortz <kent@mysql.com>
+
+- Fast mutexes is now disabled by default, but should be
+  used in Linux builds.
+
+* Mon Feb 20 2006 Kent Boortz <kent@mysql.com>
+
+- Reintroduced a max build
+- Limited testing of 'debug' and 'max' servers
+- Berkeley DB only in 'max'
+
+* Mon Feb 13 2006 Joerg Bruehe <joerg@mysql.com>
 
 - Use "-i" on "make test-force";
   this is essential for later evaluation of this log file.
 
-* Fri Dec 12 2005 Rodrigo Novo <rodrigo@mysql.com>
+* Thu Feb 09 2006 Kent Boortz <kent@mysql.com>
+
+- Pass '-static' to libtool, link static with our own libraries, dynamic
+  with system libraries.  Link with the bundled zlib.
+
+* Wed Feb 08 2006 Kristian Nielsen <knielsen@mysql.com>
+
+- Modified RPM spec to match new 5.1 debug+max combined community packaging.
+
+* Sun Dec 18 2005 Kent Boortz <kent@mysql.com>
+
+- Added "client/mysqlslap"
+
+* Mon Dec 12 2005 Rodrigo Novo <rodrigo@mysql.com>
 
 - Added zlib to the list of (static) libraries installed
 - Added check against libtool wierdness (WRT: sql/mysqld || sql/.libs/mysqld)
@@ -1023,6 +1150,10 @@ fi
 
 - Added a "make clean" between separate calls to "BuildMySQL".
 
+* Thu May 12 2005 Guilhem Bichot <guilhem@mysql.com>
+
+- Removed the mysql_tableinfo script made obsolete by the information schema
+
 * Wed Apr 20 2005 Lenz Grimmer <lenz@mysql.com>
 
 - Enabled the "blackhole" storage engine for the Max RPM
